"""
@Author: 馒头 (chocolate)
@Email: neihanshenshou@163.com
@File: __init__.py
@Time: 2023/12/9 18:00
"""

from ._BrowseTools import Browser
from ._ConfigTools import ReadConfig
from ._ConfigTools import SetEnvironment
from ._ConfigTools import get_env
from ._ConfigTools import put_env
from ._ConfigTools import set_env
from ._ConfigTools import set_env_by_file
from ._DBTools import MySQL
from ._DecoratorTools import Decorators
from ._DecoratorTools import case_attach
from ._DecoratorTools import case_desc_error
from ._DecoratorTools import case_desc_html
from ._DecoratorTools import case_desc_ok
from ._DecoratorTools import case_desc_up
from ._DecoratorTools import case_feature
from ._DecoratorTools import case_priority
from ._DecoratorTools import case_severity
from ._DecoratorTools import case_skip
from ._DecoratorTools import case_skip_if
from ._DecoratorTools import case_step
from ._DecoratorTools import case_story
from ._DecoratorTools import case_tag
from ._DecoratorTools import case_title
from ._DecoratorTools import fixture
from ._DecoratorTools import param_data
from ._DecoratorTools import param_file
from ._DecoratorTools import timer
from ._EmailTools import Email
from ._ExceptionTools import hook_exceptions
from ._ExpectTools import ExpectFormat
from ._FileTools import FileNames
from ._FileTools import FileOperate
from ._InterfaceCoverageTools import InterfaceCoverageFormat
from ._LoggerTools import LogLevels
from ._LoggerTools import logger
from ._NumberTools import NumberTools
from ._OcrTools import OcrFormat
from ._Poium import BrowserObject
from ._Poium import CSSElement
from ._Poium import Element
from ._Poium import Elements
from ._Poium import Locator
from ._Poium import Locators
from ._Poium import Page
from ._Poium import compress_image
from ._Poium import playwright
from ._Poium import processing
from ._QuoteTools import QuoteFormat
from ._RequestTools import HttpRequest
from ._RequestTools import delete
from ._RequestTools import get
from ._RequestTools import head
from ._RequestTools import options
from ._RequestTools import patch
from ._RequestTools import post
from ._RequestTools import put
from ._RequestTools import request
from ._SingletonTools import singleton
from ._TimeTools import TimeFormat



"""
@Author: 馒头 (chocolate)
@Email: neihanshenshou@163.com
@File: webdriver.py
@Time: 2023/12/9 18:00
"""

import os
import time
from time import sleep
from typing import Any

from selenium.common.exceptions import NoAlertPresentException
from selenium.common.exceptions import StaleElementReferenceException
from selenium.webdriver.common.action_chains import ActionChains

from SteamedBun import logger
from SteamedBun._Poium.selenium import BasePage


class Page(BasePage):
    """
    Implement the APIs with javascript,
    and selenium extension APIs。
    """

    def execute_script(self, js=None, *args):
        """
        Execute JavaScript scripts.
        """
        if js is None:
            raise ValueError("Please input js script")

        return self.driver.execute_script(js, *args)

    def window_scroll(self, width=None, height=None):
        """
        JavaScript API, Only support css positioning
        Setting width and height of window scroll bar.
        """
        if width is None:
            width = "0"
        if height is None:
            height = "0"
        js = f"window.scrollTo({width},{height});"
        self.execute_script(js)

    @property
    def get_title(self):
        """
        JavaScript API
        Get page title.
        """
        js = 'return document.title;'
        return self.execute_script(js)

    @property
    def get_url(self):
        """
        JavaScript API
        Get page URL.
        """
        js = "return document.URL;"
        return self.execute_script(js)

    def set_window_size(self, width=None, height=None):
        """
        selenium API
        Sets the width and height of the current window.

        :Args:
         - width: the width in pixels to set the window to
         - height: the height in pixels to set the window to

        :Usage:
            driver.set_window_size(800,600)
        """
        if width is None and height is None:
            self.driver.maximize_window()
        else:
            self.driver.set_window_size(self, width, height)

    def switch_to_frame(self, frame_reference):
        """
        selenium API
        Switches focus to the specified frame, by id, name, or webelement.
        """
        logger.warning("use page.elem.switch_to_frame() instead", DeprecationWarning, stacklevel=2)
        self.driver.switch_to.frame(frame_reference)

    def switch_to_parent_frame(self):
        """
        selenium API
        Switches focus to the parent context.
        Corresponding relationship with switch_to_frame () method.
        """
        self.driver.switch_to.parent_frame()

    @property
    def new_window_handle(self):
        """
        selenium API
        Getting a handle to a new window.
        """
        logger.warning("This method is on the verge of obsolescence", DeprecationWarning, stacklevel=2)
        all_handle = self.driver.window_handles
        return all_handle[-1]

    @property
    def current_window_handle(self):
        """
        selenium API
        Returns the handle of the current window.
        """
        logger.warning("This method is on the verge of obsolescence", DeprecationWarning, stacklevel=2)
        return self.driver.current_window_handle

    @property
    def window_handles(self):
        """
        selenium API
        Returns the handles of all windows within the current session.
        """
        logger.warning("This method is on the verge of obsolescence", DeprecationWarning, stacklevel=2)
        return self.driver.window_handles

    def switch_to_window(self, index: int) -> None:
        """
        selenium API
        Switches focus to the specified window.

        :Args:
         - window: window index. 1 represents a newly opened window (0 is the first one)

        :Usage:
            self.switch_to_window(1)
        """
        all_handles = self.driver.window_handles
        self.driver.switch_to.window(all_handles[index])

    def screenshots(self, path=None, filename=None):
        """
        selenium API
        Saves a screenshots of the current window to a PNG image file
        :param path: The path to save the file
        :param filename: The file name
        """
        if path is None:
            path = os.getcwd()
        if filename is None:
            filename = str(time.time()).split(".")[0] + ".png"
        file_path = os.path.join(path, filename)
        self.driver.save_screenshot(file_path)
        logger.info("✅ 截图当前窗口")

    def get_cookies(self):
        """
        Returns a set of dictionaries, corresponding to cookies visible in the current session.
        """
        return self.driver.get_cookies()

    def get_cookie(self, name):
        """
        Returns information of cookie with ``name`` as an object.
        """
        return self.driver.get_cookie(name)

    def add_cookie(self, cookie_dict):
        """
        Adds a cookie to your current session.
        Usage:
            add_cookie({'name' : 'foo', 'value' : 'bar'})
        """
        if isinstance(cookie_dict, dict):
            self.driver.add_cookie(cookie_dict)
        else:
            raise TypeError("Wrong cookie type.")

    def add_cookies(self, cookie_list):
        """
        Adds a cookie to your current session.
        Usage:
            cookie_list = [
                {'name' : 'foo', 'value' : 'bar'},
                {'name' : 'foo', 'value' : 'bar'}
            ]
            add_cookie(cookie_list)
        """
        if isinstance(cookie_list, list):
            for cookie in cookie_list:
                self.add_cookie(cookie)
        else:
            raise TypeError("Wrong cookie type.")

    def delete_cookie(self, name):
        """
        Deletes a single cookie with the given name.
        """
        self.driver.delete_cookie(name)

    def delete_all_cookies(self):
        """
        Delete all cookies in the scope of the session.
        Usage:
            self.delete_all_cookies()
        """
        self.driver.delete_all_cookies()

    def accept_alert(self):
        """
        selenium API
        Accept warning box.
        """
        self.driver.switch_to.alert.accept()

    def dismiss_alert(self):
        """
        selenium API
        Dismisses the alert available.
        """
        self.driver.switch_to.alert.dismiss()

    def alert_is_display(self):
        """
        selenium API
        Determines if alert is displayed
        """
        try:
            self.driver.switch_to.alert
        except NoAlertPresentException:
            return False
        else:
            return True

    @property
    def get_alert_text(self):
        """
        selenium API
        Get warning box prompt information.
        """
        return self.driver.switch_to.alert.text

    def move_to_element(self, elem):
        """
        selenium API
        Moving the mouse to the middle of an element
        """
        logger.warning("use page.elem.move_to_element() instead", DeprecationWarning, stacklevel=2)
        ActionChains(self.driver).move_to_element(elem).perform()

    def click_and_hold(self, elem):
        """
        selenium API
        Holds down the left mouse button on an element.
        """
        logger.warning("use page.elem.click_and_hold() instead", DeprecationWarning, stacklevel=2)
        ActionChains(self.driver).click_and_hold(elem).perform()

    def double_click(self, elem):
        """
        selenium API
        Double-clicks an element.
        """
        logger.warning("use page.elem.double_click() instead", DeprecationWarning, stacklevel=2)
        ActionChains(self.driver).double_click(elem).perform()

    def move_by_offset(self, x, y, click=False):
        """
        selenium API
        Moving the mouse to an offset from current mouse position.

        :Args:
         - x: X offset to move to, as a positive or negative integer.
         - y: Y offset to move to, as a positive or negative integer.
        """
        if click is True:
            ActionChains(self.driver).move_by_offset(x, y).click().perform()
            return
        ActionChains(self.driver).move_by_offset(x, y).perform()

    def release(self):
        """
        selenium API
        Releasing a held mouse button on an element.
        """
        ActionChains(self.driver).release().perform()

    def context_click(self, elem):
        """
        selenium API
        Performs a context-click (right click) on an element.
        """
        logger.warning("use page.elem.context_click() instead", DeprecationWarning, stacklevel=2)
        ActionChains(self.driver).context_click(elem).perform()

    def drag_and_drop_by_offset(self, elem, x, y):
        """
        selenium API
        Holds down the left mouse button on the source element,
           then moves to the target offset and releases the mouse button.
        :param elem: The element to mouse down.
        :param x: X offset to move to.
        :param y: Y offset to move to.
        """
        logger.warning("use page.elem.drag_and_drop_by_offset(x, y) instead", DeprecationWarning, stacklevel=2)
        ActionChains(self.driver).drag_and_drop_by_offset(elem, xoffset=x, yoffset=y).perform()

    def refresh_element(self, elem, timeout=5):
        """
        selenium API
        Refreshes the current page, retrieve elements.
        """
        logger.warning("use page.elem.refresh_element() instead", DeprecationWarning, stacklevel=2)
        try:
            timeout_int = int(timeout)
        except TypeError:
            raise ValueError("Type 'timeout' error, must be type int() ")

        for i in range(timeout_int):
            if elem is not None:
                try:
                    elem
                except StaleElementReferenceException:
                    self.driver.refresh()
                else:
                    break
            else:
                sleep(i)
        else:
            raise TimeoutError("stale element reference: element is not attached to the page document.")

    def back(self):
        """go back"""
        self.driver.back()

    def home(self):
        """press home"""
        self.driver.home()

    @staticmethod
    def sleep(sec: Any = 0.1) -> None:
        """
        Usage:
            page.sleep(seconds)
        """
        time.sleep(sec)

    def wait(self, secs: int = 5) -> None:
        """
        Implicitly wait.All elements on the page.
        Usage:
            page.wait(5)
        """
        self.driver.implicitly_wait(secs)

    def wait_script_timeout(self, time_to_wait):
        """
        Set the amount of time that the script should wait during an
           execute_async_script call before throwing an error.
        Usage:
            page.wait_script_timeout(5)
        """
        self.driver.set_script_timeout(time_to_wait)

    def wait_page_load_timeout(self, time_to_wait):
        """
        Set the amount of time to wait for a page load to complete
           before throwing an error.
        Usage:
            page.wait_page_load_timeout(5)
        """
        self.driver.set_page_load_timeout(time_to_wait)



"""
author: 馒头
email: neihanshenshou@163.com
"""


class QuoteFormat:

    @staticmethod
    def random_quote(show=False, **kwargs):
        """
        随机 毒鸡汤语录
        """
        from SteamedBun import get
        quote = get(url="https://api.oick.cn/dutang/api.php", show=show, **kwargs).json()
        return quote



"""
author: 馒头
email: neihanshenshou@163.com
"""
import os

from openpyxl import load_workbook
from openpyxl.styles import Border, Side
from openpyxl.styles import PatternFill

from SteamedBun import FileOperate


class InterfaceCoverageFormat:
    """
    接口覆盖率格式化
    """

    def __init__(self):
        ...

    @staticmethod
    def dye_color(covered_api_filename,
                  all_api_filename,
                  delete_covered=True,
                  url_title="urls",
                  method_title="methods",
                  time_title="times"):
        """
        :param covered_api_filename: api接口覆盖文件
            example - urls + 请求方法 + times
                    /api/api    get     3

        :param all_api_filename: 总的api接口文件
            example - urls + methods + times
                    /api/api    get     0
                    /api/api    post    0

        :param delete_covered: 是否删除第一个文件
        :param url_title: 第一列表头 正常是指请求路径
        :param method_title: 第二列表头 正常是指请求方法
        :param time_title: 第三列表头 正常是指请求次数

        :return 重新渲染 总的api接口文件
            example - urls + methods + times
                    /api/api    get     3(绿色)
                    /api/api    post    0(红色)
        """
        color_green = PatternFill(start_color='a4e079', end_color='a4e079', fill_type='solid')
        color_red = PatternFill(start_color='e02d09', end_color='e02d09', fill_type='solid')
        thin_border = Border(
            left=Side(style='thin', color='000000'),
            right=Side(style='thin', color='000000'),
            top=Side(style='thin', color='000000'),
            bottom=Side(style='thin', color='000000'))

        workbook1 = load_workbook(filename=covered_api_filename)
        workbook2 = load_workbook(filename=all_api_filename)
        sheet1 = workbook1.active
        sheet2 = workbook2.active
        sheet_data1 = FileOperate.read_excel(filename=covered_api_filename, sheet_name=sheet1.title)
        sheet_data2 = FileOperate.read_excel(filename=all_api_filename, sheet_name=sheet2.title)

        dict_data1 = sheet_data1.to_dict()
        dict_data2 = sheet_data2.to_dict()

        covered_count, all_api_count = 0, len(sheet_data2.values) or 0
        for index1, value1 in dict_data1.get(url_title).items():
            for index2, value2 in dict_data2.get(url_title).items():
                if value2 == value1:
                    if dict_data1.get(method_title).get(index1) == dict_data2.get(method_title).get(index2):
                        for _ in ("A", "B", "C"):
                            sheet2[f"{_}{index2 + 2}"].fill = color_green
                            sheet2[f"{_}{index2 + 2}"].border = thin_border

                        sheet2[f"C{index2 + 2}"].value = sheet1[f"C{index1 + 2}"].value
                        covered_count += 1

        if all_api_count != 0:
            from SteamedBun import NumberTools
            coverage = NumberTools.percent(
                number=covered_count * 100 / all_api_count,
                rate=2
            )
        else:
            coverage = "0 %"
        sheet2["D1"].value = "接口覆盖率"
        sheet2["D2"].value = coverage
        workbook2.save(filename=all_api_filename)
        workbook1.close()

        workbook = load_workbook(filename=all_api_filename)
        sheet = workbook.active
        sheet_data = FileOperate.read_excel(filename=all_api_filename, sheet_name=sheet.title)
        dict_data = sheet_data.to_dict()
        for each, value in dict_data.get(time_title).items():
            if str(value) == "0":
                for _ in ("A", "B", "C"):
                    sheet[f"{_}{each + 2}"].fill = color_red
                    sheet[f"{_}{each + 2}"].border = thin_border

        workbook.save(filename=all_api_filename)

        if delete_covered:
            os.remove(covered_api_filename)



"""
@Author: 馒头 (chocolate)
@Email: neihanshenshou@163.com
@File: BrowserFormat.py
@Time: 2023/12/27 21:19
"""

import os
import time

from selenium.common.exceptions import InvalidArgumentException
from selenium.webdriver import Chrome
from selenium.webdriver import Edge
from selenium.webdriver import Remote
from selenium.webdriver import Safari
from selenium.webdriver.chrome import options
from selenium.webdriver.chrome.service import Service


class ReuseBrowser(Remote):

    def __init__(self, command_executor, session_id):
        self.r_session_id = session_id
        Remote.__init__(self, command_executor=command_executor, desired_capabilities={})

    def start_session(self, capabilities, browser_profile=None):
        """
        重写start_session方法
        """
        if not isinstance(capabilities, dict):
            raise InvalidArgumentException("Capabilities must be a dict like {}")
        if browser_profile:
            if "moz:firefoxOptions" in capabilities:
                capabilities["moz:firefoxOptions"]["profile"] = browser_profile.encoded
            else:
                capabilities.update({'firefox_profile': browser_profile.encoded})

        self.caps = {} and options.Options().to_capabilities()
        self.session_id = self.r_session_id


class StartBrowser:

    def __init__(self, capabilities=None):
        self.capabilities = capabilities
        self._session_cache_path = os.path.join(os.path.dirname(__file__), "browser_session.yaml")
        self.driver = self.__start_session
        self._expect_driver = ...

    def _session_cache_get(self):
        _session_cache = {}
        if os.path.exists(self._session_cache_path):
            from SteamedBun import FileOperate
            _session_cache = FileOperate.read_file(filename=self._session_cache_path, jsonify=True, yamlify=True)

        return _session_cache

    def __start_session(self, browser_type: str = "Chrome", **kwargs):
        """
        :param browser_type: 浏览器类型 - Chrome、Edge、Safari 目前支持三类，但均需要自行下载驱动(不建议使用Edge)
        :param kwargs: executable_path - 浏览器驱动执行路径，建议配置为环境变量 一劳永逸
        :param kwargs: headless - 无头模式
        :param kwargs: wap - 是否开启wap模式
        :param kwargs: width - 自定义窗口的宽度
        :param kwargs: height - 自定义窗口的高度
        :param kwargs: scaling - 设置窗口的缩放比例(跟随系统)
        :param kwargs: options - 自定义option.Options()
        """
        from SteamedBun import logger

        _session = self._session_cache_get()
        self.capabilities = kwargs.get("caps")
        headless = kwargs.get("headless", False)
        wap = kwargs.get("wap", False)
        scaling = kwargs.get("scaling")
        if wap:
            width = kwargs.get("width", 375)
        else:
            width = kwargs.get("width", 1200)
        height = kwargs.get("height", 776)
        browser_type = browser_type.upper()

        try:
            if browser_type == "SAFARI":
                logger.warning("⚠️ ⚠️ It Seems That You Want To Use Safari Browser. "
                               "Please Don't Use By.Id To Find AN Element")
            self._expect_driver = ReuseBrowser(
                command_executor=_session.get("executor_url"),
                session_id=_session.get("session_id")
            )
            self._expect_driver.refresh()
            logger.info("✅ [Congratulation] Current Browser Session Cache Is Enable!")
        except Exception as e:
            logger.warning(f"[Ignore] {e.args[0] if len(e.args) else e.args}")
            _options = kwargs.get("options", options.Options())
            from SteamedBun import FileOperate
            if browser_type == "CHROME":
                if headless:
                    _options.headless = True
                if isinstance(self.capabilities, options.Options):
                    self.capabilities = self.capabilities.to_capabilities()
                if wap:
                    # 设置用户代理为iPhone的用户代理
                    _options.add_argument(("user-agent=Mozilla/5.0 "
                                           "(iPhone; CPU iPhone OS 14_0 like Mac OS X) "
                                           "AppleWebKit/605.1.15 (KHTML, like Gecko) Version/14.0 "
                                           "Mobile/15E148 Safari/604.1"))

                    if scaling:
                        # 可能只针对Windows系统生效
                        _options.add_argument("--high-dpi-support=1")
                        _options.add_argument(f"--force-device-scale-factor={scaling}")
                # 设置窗口大小为手机屏幕的大小(也针对web)
                _options.add_argument(f"window-size={width},{height}")
                self._expect_driver = Chrome(
                    desired_capabilities=self.capabilities,
                    options=_options,
                    service=Service("chromedriver", 0, None, None)
                )

            elif browser_type == "SAFARI":
                if headless:
                    logger.warning("⚠️ ⚠️ Safari Browser Does Not Support Headless Mode For The Time Being.")
                self._expect_driver = Safari(desired_capabilities=self.capabilities)

            elif browser_type == "EDGE":
                executable_path = kwargs.get("executable_path", "msedgedriver")
                capabilities = {}
                if headless:
                    capabilities = {"ms:edgeOptions": {
                        'args': ['--headless']
                    }}
                self._expect_driver = Edge(capabilities=capabilities, executable_path=executable_path)
            else:
                raise EnvironmentError("Only Support Browser Is [Chrome、Safari、Edge] Now")

            FileOperate.write_file(
                filename=self._session_cache_path,
                data={
                    "session_id": self._expect_driver.session_id,
                    "executor_url": self._expect_driver.service.service_url,
                    "timestamp": time.strftime("%F %T")
                }
            )
        time.sleep(0.7)
        return self._expect_driver


Browser = StartBrowser().driver



"""
@Author: 馒头 (chocolate)
@Email: neihanshenshou@163.com
@File: OcrFormat.py
@Time: 2024/2/2 21:36
"""

import requests


class OcrFormat:

    @staticmethod
    def ocr_word(filename: str = "", show=True):
        """
        识别图片中的文字
        :param filename: 图片的文件路径
        :param show: 展示结果
        :return:
        """

        response = requests.post(
            url="https://api.oioweb.cn/api/ocr",
            files={"file": open(file=filename, mode="rb", encoding=None)},
            verify=False
        )
        try:
            word = response.json().get("result", [])
        except Exception as e:
            word = [] and e

        if show and word:
            print(f"识别到的字符: {word}")
        return word



"""
@Author: 馒头 (chocolate)
@Email: neihanshenshou@163.com
@File: __init__.py
@Time: 2023/12/9 18:00
"""
from .Config import ReadConfig
from .Config import SetEnvironment
from .Config import get_env
from .Config import put_env
from .Config import set_env
from .Config import set_env_by_file



"""
@Author: 馒头 (chocolate)
@Email: neihanshenshou@163.com
@File: Config.py
@Time: 2023/12/9 18:00
"""


class BrowserObject:
    # Default browser driver
    driver = None

    # Default playwright page driver
    page = None

    # Adds a border to the action element of the operation
    show = True

    # selenium screenshot path and If you want to use, you need to set your own
    selenium_screenshot_path = None

    # playwright screenshot path and If you want to use, you need to set your own
    playwright_screenshot_path = None



import os.path
import pathlib
import sys
import time
from typing import Any, Dict, List, Optional, TypedDict, Union, Pattern

import allure
from playwright._impl._element_handle import ElementHandle  # noqa
from playwright._impl._helper import (  # noqa
    KeyboardModifier,
    MouseButton,
    locals_to_params,
)
from playwright._impl._js_handle import Serializable, JSHandle  # noqa

from SteamedBun import logger
from SteamedBun._Poium.config import BrowserObject

if sys.version_info >= (3, 8):  # pragma: no cover
    from typing import Literal
else:  # pragma: no cover
    from typing_extensions import Literal


class FilePayload(TypedDict):
    name: str
    mimeType: str
    buffer: bytes


class FloatRect(TypedDict):
    x: float
    y: float
    width: float
    height: float


class Position(TypedDict):
    x: float
    y: float


def _auto_screenshot_with_playwright(elem, driver, describe=""):
    from SteamedBun import processing, case_attach

    _tmp_filename = f"{describe or 'playwright'}_screenshot_{int(time.time() * 1000)}.png"
    _tmp_filename = os.path.join(BrowserObject.playwright_screenshot_path, _tmp_filename)
    driver.screenshot(path=_tmp_filename)
    position_elem = elem.bounding_box()
    elem_x, elem_y = position_elem.get("x", 0), position_elem.get("y", 0)
    viewport_size = driver.viewport_size
    viewport_width, viewport_height = viewport_size.get("width", 0), viewport_size.get("height", 0)

    processing(
        filename=_tmp_filename,
        w=elem_x / viewport_width,
        h=elem_y / viewport_height
    )

    case_attach(
        body=open(file=_tmp_filename, mode="rb").read(),
        name=_tmp_filename,
        attachment_type=allure.attachment_type.PNG
    )


class Page(object):
    """
    Page Object pattern.
    """

    def __init__(self, page):
        """
        :param page: `playwright.sync_api.Page`
        Root URI to base any calls to the ``PageObject.get`` method. If not defined
        in the constructor it will try and look it from the webdriver object.
        """
        self.page = page

    @staticmethod
    def sleep(secs=1):
        logger.info(f"等待 {secs} s.")
        return time.sleep(secs)

    def screenshot(self, filename=""):
        if not filename:
            filename = f"{int(time.time())}.png"
        logger.info(f"截图: {filename}")
        return self.page.screenshot(path=filename)

    def open(self, uri):
        """
        open uri
        :param uri:  URI to GET, based off of the root_uri attribute.
        :return:
        """
        self.page.goto(uri)

    def goto(self, uri):
        """
        goto uri
        :param uri:  URI to GET, based off of the root_uri attribute.
        :return:
        """
        self.open(uri=uri)

    def get(self, uri):
        """
        get uri
        :param uri:  URI to GET, based off of the root_uri attribute.
        :return:
        """
        self.open(uri=uri)

    def close(self):
        """
        Close the current Window Page
        """
        self.page.close()


class Locator:
    """
    Returns an element object
    """

    def __init__(
            self,
            selector: str,
            describe: str = "",
            position: Position = None,
            modifiers: List[KeyboardModifier] = None,
            delay: float = None,
            timeout: float = 1.0,
            force: bool = None,
            no_wait_after: bool = None,
            trial: bool = None,
            index: int = 0):
        self.selector = selector
        self.desc = describe
        self.position = position
        self.modifiers = modifiers
        self.delay = delay
        self.timeout = timeout * 1000
        self.force = force
        self.no_wait_after = no_wait_after
        self.trial = trial
        self.index = index

    @property
    def find(self):
        self.driver.wait_for_load_state("networkidle")
        elems = []
        for _ in range(3):
            try:
                elems = self.driver.locator(self.selector).all()
                if len(elems) != 0:
                    break
            except Exception as e:
                logger.error(f"❌ {e}")
            _ = self.timeout / 1000
            time.sleep(_)
            logger.info(f"等待 {_} s.")
        if not len(elems):
            logger.error(f"❌ 未找到对应元素. {self.desc}")
        elem = elems[self.index]
        elem.evaluate('node => node.style.cssText="border:solid 2px blue"')
        time.sleep(0.02)
        elem.evaluate('node => node.style.cssText="border:solid 2px green"')
        time.sleep(0.02)
        elem.evaluate('node => node.style.cssText="border:solid 2px red"')
        time.sleep(0.01)
        elem.evaluate('node => node.style.cssText="border:solid 2px none"')

        if BrowserObject.playwright_screenshot_path:
            _auto_screenshot_with_playwright(elem=elem, driver=self.driver, describe=self.desc)

        if self.desc == "":
            logger.info(f"✨ Find element.")
        else:
            logger.info(f"✨ Find element: {self.desc}.")
        return elem

    def __get__(self, instance, owner):
        if instance is None:
            return None

        self.driver = instance.page
        return self

    def __set__(self, instance, value):
        elem = self.__get__(instance, instance.__class__)
        elem.fill(value)

    def all_inner_texts(self) -> List[str]:
        """
        Returns an array of node.innerText values for all matching nodes.
        :return:
        """
        return self.find.all_inner_texts()

    def all_text_contents(self) -> List[str]:
        """
        Returns an array of node.textContent values for all matching nodes.
        :return:
        """
        return self.find.all_text_contents()

    def bounding_box(self) -> Optional[FloatRect]:
        """
        This method returns the bounding box of the element, or null if the element is not visible.
        :return:
        """
        return self.find.bounding_box(timeout=self.timeout)

    def fill(self, value: str) -> None:
        """
        Text input
        :param value:
        :return:
        """
        return self.find.fill(
            value=value,
            timeout=self.timeout,
            no_wait_after=self.no_wait_after,
            force=self.force,
        )

    def check(self) -> None:
        """
        Check the checkbox.
        :return:
        """
        return self.find.check(
            position=self.position,
            timeout=self.timeout,
            force=self.force,
            no_wait_after=self.no_wait_after,
            trial=self.trial,
        )

    def uncheck(self) -> None:
        """
        Uncheck by input <label>
        :return:
        """
        return self.find.uncheck(
            position=self.position,
            timeout=self.timeout,
            force=self.force,
            no_wait_after=self.no_wait_after,
            trial=self.trial
        )

    def select_option(
            self,
            value: Union[str, List[str]] = None,
            index: Union[int, List[int]] = None,
            label: Union[str, List[str]] = None,
            element: Union["ElementHandle", List["ElementHandle"]] = None,
    ) -> List[str]:
        """
        Selects one or multiple options in the <select> element.
        :return:
        """
        return self.find.select_option(
            value=value,
            index=index,
            label=label,
            element=element,
            timeout=self.timeout,
            no_wait_after=self.no_wait_after,
            force=self.force
        )

    def select_text(self) -> None:
        """
        This method waits for actionability checks,
         then focuses the element and selects all its text content.
        :return:
        """
        return self.find.select_text(
            force=self.force,
            timeout=self.timeout
        )

    async def set_checked(self, checked: bool) -> None:
        """
        This method checks or unchecks an element.
        :param checked:
        :return:
        """
        return self.find.set_checked(
            checked=checked,
            position=self.position,
            timeout=self.timeout,
            force=self.force,
            no_wait_after=self.no_wait_after,
            trial=self.trial,
        )

    def click(self, click_count: int = None, button: MouseButton = None) -> None:
        """
        click
        :param click_count:
        :param button:
        :return:
        """
        return self.find.click(
            modifiers=self.modifiers,
            position=self.position,
            delay=self.delay,
            button=button,
            click_count=click_count,
            timeout=self.timeout,
            force=self.force,
            no_wait_after=self.no_wait_after,
            trial=self.trial,
        )

    def enter(self):
        """
        模拟回车操作
        """
        return self.find.press("Enter")

    def send_keywords(self, value="", enter=False, clear=False):
        """
        send keywords、input
        """
        if clear:
            self.find.fill(value="")
        self.find.fill(value=value)
        if enter:
            self.find.press("Enter")

        return None

    def send_keyboards(self, key="Enter"):
        """
        send Mouse Key Boards
        """

        return self.find.press(key=key)

    def count(self) -> int:
        """
        Returns the number of elements matching given selector.
        :return:
        """
        return self.find.count()

    def dblclick(self, button: MouseButton = None) -> None:
        """
        double click
        :param button:
        :return:
        """
        return self.find.dblclick(
            modifiers=self.modifiers,
            position=self.position,
            delay=self.delay,
            button=button,
            timeout=self.timeout,
            force=self.force,
            no_wait_after=self.no_wait_after,
            trial=self.trial,
        )

    def drag_to(
            self,
            target: "Locator",
            source_position: Position = None,
            target_position: Position = None
    ) -> None:
        """
        Locator of the element to drag to.
        param source_position:
        param target_position:
        param target:
        :return:
        """
        return self.find.drag_to(
            target=target,
            force=self.force,
            no_wait_after=self.no_wait_after,
            timeout=self.timeout,
            trial=self.trial,
            source_position=source_position,
            target_position=target_position,
        )

    def element_handle(self) -> ElementHandle:
        """
        Resolves given locator to the first matching DOM element
        :return:
        """
        return self.find.element_handle(timeit=self.timeout)

    def element_handles(self) -> List[ElementHandle]:
        """
        Resolves given locator to all matching DOM elements.
        :return:
        """
        return self.find.element_handles()

    @property
    def first(self) -> "Locator":
        """
        Returns locator to the first matching element.
        :return:
        """
        return self.find.first

    @property
    def last(self) -> "Locator":
        """
        Returns locator to the last matching element.
        :return:
        """
        return self.find.last

    def nth(self, index: int) -> "Locator":
        """
        Returns locator to the n-th matching element. It's zero based, nth(0) selects the first element.
        :param index:
        :return:
        """
        return self.find.nth(index=index)

    @property
    def page(self) -> "Page":
        """
        A page this locator belongs to.
        :return:
        """
        return self.find.page()

    def filter(
            self,
            has_text: Union[str, Pattern[str]] = None,
            has: "Locator" = None,
    ) -> "Locator":
        """
        This method narrows existing locator according to the options.
        :param has_text:
        :param has:
        :return:
        """
        return self.find.filter(has_text=has_text, has=has)

    def get_attribute(self, name: str):
        """
        Returns element attribute value.
        :param name:
        :return:
        """
        return self.find.get_attribute(name=name, timeout=self.timeout)

    def highlight(self) -> None:
        return self.find.highlight()

    def hover(self) -> None:
        """
        Hover over element
        :return:
        """
        return self.find.hover(
            modifiers=self.modifiers,
            position=self.position,
            timeout=self.timeout,
            force=self.force,
            trial=self.trial,
        )

    def dispatch_event(self, typo: str, event_init: Dict = None) -> None:
        """
        Programmatic click
        :param typo:
        :param event_init:
        :return:
        """
        return self.find.dispatch_event(
            type=typo,
            eventInit=event_init,
            timeout=self.timeout,
        )

    def evaluate(self, expression: str, arg: Serializable = None) -> Any:
        """
        This method passes this handle as the first argument to expression.
        :param expression:
        :param arg:
        :return:
        """
        return self.find.evaluate(
            expression=expression,
            arg=arg,
            timeout=self.timeout
        )

    def evaluate_all(self, expression: str, arg: Serializable = None) -> Any:
        """
        The method finds all elements matching the specified locator and
        passes an array of matched elements as a first argument to expression.
        Returns the result of expression invocation.
        :param expression:
        :param arg:
        :return:
        """
        params = locals_to_params(locals())
        return self.find.evaluate_all(
            expression=expression,
            arg=arg
        )

    def evaluate_handle(self, expression: str, arg: Serializable = None) -> "JSHandle":
        """
        This method passes this handle as the first argument to expression.
        :param expression:
        :param arg:
        :return:
        """
        return self.find.evaluate_handle(
            expression=expression,
            arg=arg,
            timeout=self.timeout
        )

    def tap(self) -> None:
        """
        This method taps the element.
        :return:
        """
        return self.find.tap(
            modifiers=self.modifiers,
            position=self.position,
            timeout=self.timeout,
            force=self.force,
            no_wait_after=self.no_wait_after,
            trial=self.trial,
        )

    def text_content(self) -> Optional[str]:
        """
        Returns the node.textContent.
        :return:
        """
        return self.find.text_content(timeout=self.timeout)

    def type(self, text: str) -> None:
        """
        Type into the field character by character, as if it was a user with a real keyboard.
        :param text:
        :return:
        """
        return self.find.type(
            text=text,
            delay=self.delay,
            timeout=self.timeout,
            no_wait_after=self.no_wait_after,
        )

    def press(self, key: str) -> None:
        """
        Keys and shortcuts
        :param key:
        :return:
        """
        return self.find.press(
            key=key,
            delay=self.delay,
            timeout=self.timeout,
            no_wait_after=self.no_wait_after,
        )

    def set_input_files(
            self,
            files: Union[
                str,
                pathlib.Path,
                FilePayload,
                List[Union[str, pathlib.Path]],
                List[FilePayload],
            ]
    ) -> None:
        """
        Upload files
        :param files:
        :return:
        """
        return self.find.set_input_files(
            files=files,
            timeout=self.timeout,
            no_wait_after=self.no_wait_after,
        )

    def focus(self) -> None:
        """
        Focus element
        :return:
        """
        return self.find.focus(timeout=self.timeout)

    def inner_html(self) -> str:
        """
        Returns the element.innerHTML.
        :return:
        """
        return self.find.inner_html(timeout=self.timeout)

    def inner_text(self) -> str:
        """
        Returns the element.innerText.
        :return:
        """
        return self.find.inner_text(timeout=self.timeout)

    async def input_value(self) -> str:
        """
        Returns input.value for the selected <input> or <textarea> or <select> element.
        :return:
        """
        return self.find.input_value(timeout=self.timeout)

    def is_checked(self) -> bool:
        """
        Returns whether the element is checked.
        Throws if the element is not a checkbox or radio input.
        :return:
        """
        return self.find.is_checked(timeout=self.timeout)

    def is_disabled(self) -> bool:
        """
        Returns whether the element is disabled, the opposite of enabled.
        :return:
        """
        return self.find.is_disabled(timeout=self.timeout)

    def is_editable(self) -> bool:
        """
        Returns whether the element is editable.
        :return:
        """
        return self.find.is_editable(timeout=self.timeout)

    def is_enabled(self) -> bool:
        """
        Returns whether the element is enabled.
        :return:
        """
        return self.find.is_enabled(timeout=self.timeout)

    def is_hidden(self) -> bool:
        """
        Returns whether the element is hidden, the opposite of visible.
        :return:
        """
        return self.find.is_hidden(timeout=self.timeout)

    def is_visible(self) -> bool:
        """
        Returns whether the element is visible.
        :return:
        """
        return self.find.is_visible(timeout=self.timeout)

    def screenshot(
            self,
            typo: Literal["jpeg", "png"] = None,
            path: Union[str, pathlib.Path] = None,
            quality: int = None,
            omit_background: bool = None,
            animations: Literal["allow", "disabled"] = None,
            caret: Literal["hide", "initial"] = None,
            scale: Literal["css", "device"] = None,
            mask: List["Locator"] = None,
    ) -> bytes:
        """
        This method captures a screenshot of the page
        :param typo:
        :param path:
        :param quality:
        :param omit_background:
        :param animations:
        :param caret:
        :param scale:
        :param mask:
        :return:
        """
        return self.find.screenshot(
            timeout=self.timeout,
            type=typo,
            path=path,
            quality=quality,
            omit_background=omit_background,
            animations=animations,
            caret=caret,
            scale=scale,
            mask=mask,
        )

    def scroll_into_view_if_needed(self) -> None:
        """
        This method waits for actionability checks, then tries to scroll element into view,
         unless it is completely visible as defined by IntersectionObserver's ratio.
        :return:
        """
        return self.find.scroll_into_view_if_needed(timeout=self.timeout)

    def wait_for(self, state: Literal["attached", "detached", "hidden", "visible"] = None) -> None:
        """
        Returns when element specified by locator satisfies the state option.
        :param state:
        :return:
        """
        return self.find.wait_for(
            timeout=self.timeout,
            state=state,
        )


class Locators:
    """
    Returns elements object
    """

    def __init__(
            self,
            selector: str,
            describe: str = "",
            position: Position = None,
            modifiers: List[KeyboardModifier] = None,
            delay: float = None,
            timeout: float = 1.0,
            force: bool = None,
            no_wait_after: bool = None,
            trial: bool = None,
            index: int = 0,
            has_context: bool = bool(False)):
        self.selector = selector
        self.desc = describe
        self.position = position
        self.modifiers = modifiers
        self.delay = delay
        self.timeout = timeout * 1000
        self.force = force
        self.no_wait_after = no_wait_after
        self.trial = trial
        self.index = index
        self.has_context = has_context

    def find(self, context):
        context.wait_for_load_state("load")
        elems = []
        for _ in range(3):
            try:
                elems = context.locator(self.selector).all()
                if len(elems) != 0:
                    break
            except Exception as e:
                logger.error(f"❌ {e}")
            time.sleep(self.timeout / 1000)
        for elem in elems:
            elem.evaluate('node => node.style.cssText="border:solid 2px blue"')
            time.sleep(0.02)
            elem.evaluate('node => node.style.cssText="border:solid 2px green"')
            time.sleep(0.02)
            elem.evaluate('node => node.style.cssText="border:solid 2px red"')
            time.sleep(0.01)
            elem.evaluate('node => node.style.cssText="border:solid 2px none"')

            if BrowserObject.playwright_screenshot_path:
                _auto_screenshot_with_playwright(elem=elem, driver=context, describe=self.desc)

        if self.desc == "":
            logger.info(f"✨ Find elements.")
        else:
            logger.info(f"✨ Find elements: {self.desc}.")
        return elems

    def __get__(self, instance, owner, context=None):
        if not instance:
            return None

        if not context and self.has_context:
            return lambda ctx: self.__get__(instance, owner, context=ctx)

        if not context:
            context = instance.page

        return self.find(context)

    def __set__(self, instance, value):
        elems = self.__get__(instance, instance.__class__)
        [elem.fill(value) for elem in elems]



"""
@Author: 馒头 (chocolate)
@Email: neihanshenshou@163.com
@File: _ExceptionTools.py
@Time: 2023/12/22 19:58
"""


def hook_exceptions(exceptions: (tuple, set, list) = None):
    """
    捕获异常装饰器
    若未指定捕获的非基类异常, 则默认是忽略其它异常捕获
    example: 若 exception_types = [AssertionError], 则只会捕获这个方法中抛出的AssertionError, 而会忽略类似于ValueError等其它异常
    :param exceptions: 指定的异常
    :return: None
    """
    if isinstance(exceptions, (tuple, set, list)):
        exception_types = tuple(exceptions)
    elif isinstance(exceptions, type):
        exception_types = (exceptions,)
    elif exceptions is None:
        exception_types = tuple()

    else:
        raise Exception("参数类型错误: 需要将多个异常类以元组的形式上传")

    def decorator(func):
        def wrapper(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except exception_types as e:
                raise e
            except Exception as e2:
                if type(e2) not in exception_types:
                    exception_types_name = [_.__name__ for _ in exception_types]
                    print(f"""
---------------------------------------------                    
当前的执行方法名称: {func.__name__}
未捕获指定异常类型: {exception_types_name}
或是故意忽略该异常: {type(e2).__name__}
--------------------------------------------- 
                    """)

        return wrapper

    return decorator



"""
@Author: 馒头 (chocolate)
@Email: neihanshenshou@163.com
@File: EmailFormat.py
@Time: 2023/12/9 18:00
"""

import smtplib
import time
from email.header import Header
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.utils import parseaddr, formataddr


class Email:

    def __init__(self, receiver=None):
        """
        一些初始化的信息
            Email(
                receiver=['478693360@qq.com']
            ).bedeck_report_html(
                data={"total": 100, "passed": 95, "fail": 3, "error": 1, "skip": 2, "rate": 95, "duration": "13秒"}
            )
        """

        self.smtp = 'smtp.163.com'  # 邮箱服务器地址
        self.username = 'neihanshenshou@163.com'  # 发件人地址
        self.password = "YWYGLQNOPLWNNJJY"  # 授权码

        self.subject = '自动化测试报告'  # 邮件主题
        self.sender = 'neihanshenshou@163.com'  # 发件人
        if isinstance(receiver, list):
            self.receiver = receiver
        elif isinstance(receiver, str):
            self.receiver = receiver.split(",")

    @staticmethod
    def _format_addr(s):
        name, addr = parseaddr(s)
        return formataddr((Header(name, 'utf-8').encode(), addr))

    def message_init(self, html):
        """
        对即将发送的内容进行初始化
        :param html: 发送邮件正文内容
        :return:
        """
        message = MIMEMultipart()  # 内容接收池
        message['subject'] = Header(self.subject, 'utf-8')
        message['From'] = self._format_addr('馒头 <%s>' % self.sender)
        message['To'] = ', '.join(self.receiver)
        # 解决乱码，html是html格式的字符串
        message_content = MIMEText(html, _subtype='html', _charset='utf-8')
        # 邮件的正文内容
        message.attach(message_content)

        return message

    def send_email(self, html):
        """
        发送邮件
        :param html: 邮件正文内容
        """
        try:
            # 连接邮箱的服务器及端口号
            smtp_obj = smtplib.SMTP(self.smtp, 25)  # smtp服务器端口默认是25
            # 登录邮箱：用户名及密码
            smtp_obj.login(self.username, self.password)
            # 发送邮件
            smtp_obj.sendmail(self.sender, self.receiver, self.message_init(html).as_string())
            print('邮件发送成功')
            smtp_obj.quit()  # 关闭服务器
        except Exception as e:
            raise Exception('邮件发送失败：{}'.format(e))

    @staticmethod
    def _rich_txt_html(url, total, passed, fail, error, skip, color, rate, duration):
        return f"""
            <table border="1px" width="330" align="center" cellspacing="0">
                <h1 align="center">自动化测试报告【{time.strftime("%F")}】</h1>
                <tr align="center">
                    <td>维度统计</td>
                    <td><a href="{url}" style="text-decoration:none">详情(点击查看)</a></td>
                </tr>
                <tr align="center">
                    <td>用例总数</td>
                    <td><b>{total}</b></td>
                </tr>
                <tr align="center">
                    <td>用例通过</td>
                    <td style="color: green"><b>{passed}</b></td>
                </tr>
                <tr align="center">
                    <td>用例失败</td>
                    <td style="color: orange"><b>{fail}</b></td>
                </tr>
                <tr align="center">
                    <td>用例跳过</td>
                    <td style="color: gray"><b>{skip}</b></td>
                </tr>
                <tr align="center">
                    <td>用例错误</td>
                    <td style="color: red"><b>{error}</b></td>
                </tr>
                <tr align="center">
                    <td>成功率</td>
                    <td style="color: {color}"><b>{rate}%</b></td>
                </tr>
                <tr align="center">
                    <td>执行时长</td>
                    <td><b>{duration}</b></td>
                </tr>
            </table>
        """

    def bedeck_report_html(self, data: dict):
        """
            修饰测试报告html文案
        Args:
            data: {
                    "url": "http://localhost:7021/",
                    "total": 10,
                    "passed": 9,
                    "fail": 1,
                    "skip": 0,
                    "error": 0,
                    "rate": "90%",
                    "duration": "1秒"
                }
        Returns:

        """
        report_url = data.get("url", "https://github.com/neihanshenshou/SteamedBun/blob/master/README.md")
        total = data.get("total", 0)
        passed = data.get("pass", 0)
        fail = data.get("fail", 0)
        skip = data.get("skip", 0)
        error = data.get("error", 0)
        rate = data.get("rate", 0) or passed / total * 100
        duration = data.get("duration", "未统计时间")
        if rate == 100:
            color = "green"
        elif rate < 90:
            color = "red"
        else:
            color = "orange"

        message = self._rich_txt_html(
            url=report_url,
            total=total,
            passed=passed,
            fail=fail,
            error=error,
            skip=skip,
            color=color,
            rate=f"{rate:.2f}",
            duration=duration
        )
        self.send_email(html=message)
        return message



"""
author: 馒头
email: neihanshenshou@163.com
"""
from pymysql.connections import Connection
from pymysql.cursors import DictCursor


class MySQL:

    def __init__(self, user="root", password="123456", database="mantou", host="127.0.0.1", port=3306, typo=DictCursor):
        self.conn = Connection(
            user=user,
            password=password,
            host=host,
            port=port,
            database=database,
            cursorclass=typo
        )

    def __enter__(self):
        return self

    def query(self, sql=None):
        cursor = self.conn.cursor()
        cursor.execute(query=sql or "select * from lick_dog")
        data = cursor.fetchall()
        cursor.close()
        return [each for each in data]

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.conn.close()



"""
@Author: 馒头 (chocolate)
@Email: neihanshenshou@163.com
@File: Singleton.py
@Time: 2023/12/9 18:00
"""


def singleton(cls):
    instance = dict()

    def inner(*args, **kwargs):
        if not instance.get(cls):
            obj = cls(*args, **kwargs)
            instance[cls] = obj
        else:
            obj = instance[cls]
        return obj

    return inner


class SingletonExample:
    instance = {}

    def __init__(self):
        self.A = "111"

    def __new__(cls, *args, **kwargs):
        if cls.instance is None:
            cls.instance = super().__new__(cls, *args, **kwargs)

        return cls.instance



"""
@Author: 馒头 (chocolate)
@Email: neihanshenshou@163.com
@File: Config.py
@Time: 2023/12/9 18:00
"""
import configparser
import os
from configparser import ConfigParser
from pathlib import Path


class ReadConfig:

    def __init__(self, config_path: str = ""):
        self.cfg = ConfigParser()
        self.cfg.read(filenames=config_path, encoding="utf-8")

    def get_section(self, key):
        return dict(self.cfg.items(key))


def _get_env(key="env"):
    return os.environ.get(key)


def _set_env(env: str):
    return os.environ.setdefault(key="env", value=env)


def _put_env(env: str):
    return os.environ.__setitem__(key="env", value=env)


def _read_ini(ini_path=None) -> dict:
    ph = os.path.join(os.path.dirname(__file__), "config.ini")
    if not ini_path and not os.path.exists(ph):
        return {
            "current_env": "test",
            "enable_envs": [
                "test",
                "online"
            ]
        }
    else:
        cp = ReadConfig(ini_path or ph)
        try:
            tmp_dict = cp.get_section(key="environments")
        except configparser.NoSectionError:
            tmp_dict = cp.get_section("envs")
        return tmp_dict


class SetEnvironment:
    instance = None

    def __new__(cls, *args, **kwargs):
        if cls.instance is None:
            cls.instance = super().__new__(cls)
        return cls.instance

    def __init__(self, env_path=None, put_flag=False):
        cur_path = None
        if isinstance(env_path, (str, Path)) and env_path:

            from SteamedBun import FileOperate
            from SteamedBun import logger

            env_path = os.path.join(env_path)
            if not (env_path.endswith(".ini") or env_path.endswith(".config")):
                raise FileExistsError(f"i hope accept .ini or .config instead of {env_path.split('.')[-1]}")
            else:
                cur_path = os.path.join(os.path.dirname(__file__), "config.ini")
                if not os.path.exists(cur_path):
                    FileOperate.write_file(filename=cur_path, data=FileOperate.read_file(filename=env_path))
                    logger.info("相关环境配置写入默认配置当中, config.ini自此生效")
                if FileOperate.read_file(filename=cur_path) != FileOperate.read_file(filename=env_path):
                    FileOperate.write_file(filename=cur_path, data=FileOperate.read_file(filename=env_path))

        final_ini = _read_ini(ini_path=cur_path)
        self.current_env = final_ini["current_env"]
        self.enable_envs = final_ini["enable_envs"]
        if not _get_env() or _get_env() != self.current_env:
            _set_env(self.current_env)
            if put_flag:
                _put_env(self.current_env)

        if _get_env() not in _read_ini(ini_path=cur_path)["enable_envs"]:
            raise EnvironmentError(f"当前运行环境:{_get_env()} 不在可执行环境内: {self.enable_envs}")
        elif _get_env() != self.current_env:
            self.current_env = _get_env()


set_env_by_file = SetEnvironment
get_env = _get_env
set_env = _set_env
put_env = _put_env



"""
@Author: 馒头 (chocolate)
@Email: neihanshenshou@163.com
@File: Logger.py
@Time: 2023/12/9 18:00
"""

import logging
import sys
from typing import Callable

from colorama import Fore, Style


class LogLevels:
    """
    日志级别
    """
    # 系统默认级别
    CRITICAL = 50
    FATAL = CRITICAL
    ERROR = 40
    WARNING = 30
    WARN = WARNING
    INFO = 20
    DEBUG = 10
    NOTSET = 0

    # 定义自定义级别
    STEP_LEVEL_NUM = 23


logging.addLevelName(LogLevels.STEP_LEVEL_NUM, "STEP")


class StepLogger(logging.Logger):
    # 创建一个自定义日志记录方法
    def step(self, msg, *args, **kwargs):
        if self.isEnabledFor(LogLevels.STEP_LEVEL_NUM):
            self._log(LogLevels.STEP_LEVEL_NUM, msg, args, **kwargs)


# 将自定义方法添加到Logger类中
logging.setLoggerClass(StepLogger)

_logger = logging.getLogger(__name__)


class _Style:
    _GREEN = Fore.GREEN
    _BLACK = Fore.BLACK
    _RED = Fore.RED
    _ORIGIN = Style.RESET_ALL
    _BOLD = Style.BRIGHT

    stream_format = (
        f'{_ORIGIN}{_GREEN}%(asctime)s '
        f'{_ORIGIN}{_BOLD}| %(filename)s'
        f'{_BLACK}{_BOLD} | %(lineno)s | %(levelname)s {_BOLD}| '
        f'{_BOLD}%(message)s {_ORIGIN}')

    stream_level = 'DEBUG'
    file_format = '%(asctime)s - %(filename)s - %(lineno)s - %(levelname)s - %(message)s'
    file_level = 'DEBUG'

    BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE = range(8)

    RESET_SEQ = "\033[0m"
    COLOR_SEQ = "\033[1;%dm"
    BOLD_SEQ = "\033[1m"

    COLORS = {
        'WARNING': YELLOW,
        'INFO': CYAN,
        'STEP': GREEN,
        'DEBUG': BLUE,
        'CRITICAL': MAGENTA,
        'ERROR': RED
    }


class ColoredFormatter(logging.Formatter):
    def __init__(self, msg, use_color=True):
        logging.Formatter.__init__(self, msg)
        self.use_color = use_color

    def format(self, record):
        level_name = record.levelname
        if self.use_color and level_name in _Style.COLORS:
            level_name_color = _Style.COLOR_SEQ % (30 + _Style.COLORS[level_name]) + level_name + _Style.RESET_SEQ
            record.levelname = level_name_color

        # 移除 ANSI 转义序列
        message = logging.Formatter.format(self, record)
        record.levelname = level_name

        return message


class ColoredLogger(logging.Logger):

    def __init__(self, name):
        logging.Logger.__init__(self, name, logging.DEBUG)
        color_formatter = ColoredFormatter(_Style.stream_format)

        console = logging.StreamHandler()
        console.setFormatter(color_formatter)

        self.addHandler(console)
        return


def _set_filename(filename=None, level="DEBUG") -> None:
    """
    设置日志存储目录
    :param filename: 日志存储目录
    :return: None
    """
    # 文件日志中打印 debug 级别的日志
    file_handler = logging.FileHandler(filename=filename, encoding="utf-8")
    file_handler.setLevel(level=level or _Style.file_level)
    file_handler.setFormatter(fmt=logging.Formatter(fmt=_Style.file_format))
    # 拒绝添加重复句柄
    _logger.addHandler(hdlr=file_handler)


def _stream_setting():
    _logger.setLevel(LogLevels.INFO)
    # 终端只打印 info 级别的日志
    _stream_handler = logging.StreamHandler(stream=sys.stdout)
    _stream_handler.setLevel(level=_Style.stream_level)

    color_formatter = ColoredFormatter(_Style.stream_format)
    _stream_handler.setFormatter(fmt=color_formatter)

    # 拒绝添加重复句柄
    _logger.addHandler(hdlr=_stream_handler)


class Logger:
    _stream_setting()
    _logger.setFilename = _set_filename
    _logger.step = _logger.step

    def __init__(self):
        self.setFilename: Callable[..., None] = self.__set_filename
        self.setLevel: Callable[..., None] = self.__set_stream_level
        self.step: Callable[..., None] = self.step

    @classmethod
    def __set_stream_level(cls, level: str):
        logging.StreamHandler().setLevel(level=level)

    @classmethod
    def __set_filename(cls, filename: str, level="DEBUG"):
        _set_filename(filename=filename, level=level)

    @staticmethod
    def debug(msg, *args, **kwargs):
        _logger.debug(msg=str(msg), *args, **kwargs)

    @staticmethod
    def info(msg, *args, **kwargs):
        _logger.info(msg=str(msg), *args, **kwargs)

    @staticmethod
    def warning(msg, *args, **kwargs):
        _logger.warning(msg=str(msg), *args, **kwargs)

    @staticmethod
    def error(msg, *args, **kwargs):
        _logger.error(msg=str(msg), *args, **kwargs)

    @staticmethod
    def critical(msg, *args, **kwargs):
        _logger.critical(msg=str(msg), *args, **kwargs)

    @staticmethod
    def step(msg, *args, **kwargs):
        _logger.step(msg=str(msg), *args, **kwargs)


logger: Logger = _logger



"""
author: 馒头
email: neihanshenshou@163.com
"""

from typing import Any, Literal
from typing import Callable
from typing import Optional
from typing import Sequence
from typing import TypeVar
from typing import Union

import allure
import pytest

_ScopeName = Literal["session", "package", "module", "class", "function"]
FixtureFunction = TypeVar("FixtureFunction", bound=Callable[..., object])


def fixture(
        fixture_function: Optional[FixtureFunction] = None,
        scope: "Union[_ScopeName, Callable[[str, []], _ScopeName]]" = "function",
        params=None,
        autouse: bool = False,
        ids: Optional[
            Union[Sequence[Optional[object]], Callable[[Any], Optional[object]]]
        ] = None,
        name: Optional[str] = None):
    return pytest.fixture(
        fixture_function=fixture_function,
        scope=scope,
        params=tuple(params) if params is not None else None,
        autouse=autouse,
        ids=None if ids is None else ids if callable(ids) else tuple(ids),
        name=name
    )


def case_title(title):
    """
    用例标题文案
    """

    def inner(func):
        return allure.title(test_title=title)(func)

    return inner


def case_step(desc):
    """
    用例步骤描述
    :param desc: 步骤描述
    """

    from SteamedBun import logger
    logger.step(msg=desc)
    return allure.step(title=desc)


def case_severity(level):
    """
    用例优先级、重要级别
    """

    def inner(func):
        return allure.severity(severity_level=level)(func)

    return inner


def case_tag(*tags):
    """
    用例标签
    """

    def inner(func):
        return allure.tag(*tags)(func)

    return inner


def case_feature(*features):
    """
    用例特性
    """

    def inner(func):
        return allure.feature(*features)(func)

    return inner


def case_story(*stories):
    """
    用例描述
    """

    def inner(func):
        return allure.story(*stories)(func)

    return inner


def case_attach(body, name=None, attachment_type=allure.attachment_type.TEXT):
    """
    用例附件
    """
    return allure.attach(
        body=body,
        name=name,
        attachment_type=attachment_type
    )


def case_skip(reason: str = ...):
    """
    用例 任意情况下跳过执行
    """
    return pytest.mark.skip(reason=reason)


def case_skip_if(condition, *conditions, reason):
    """
    用例 满足条件跳过执行
    """
    return pytest.mark.skipif(condition=condition, conditions=conditions, reason=reason)


def case_desc_html(content):
    """
    用例描述文案默认提示
    """

    def inner(func):
        return allure.description_html(test_description_html=content)(func)

    return inner


def case_desc_ok(content: str):
    """
    用例描述文案绿色提示
    """

    def inner(func):
        new_content = ('<h2>'
                       '<span>Tip:【用例正常运行】</span>'
                       '</h2>')
        for index, each in enumerate(content.split(';')):
            new_content += f'<h3 style="color: #2bbf02">{index + 1}. {each}</h3>'
        return allure.description_html(test_description_html=new_content)(func)

    return inner


def case_desc_error(content: str):
    """
    用例描述文案红色提示
    """

    def inner(func):
        new_content = ('<h2>'
                       '<span>Tip:【用例发现缺陷】</span>'
                       '</h2>')
        for index, each in enumerate(content.split(';')):
            new_content += f'<h3 style="color: #ff0011">{index + 1}. {each}</h3>'
        return allure.description_html(test_description_html=new_content)(func)

    return inner


def case_desc_up(content: str):
    """
    用例描述文案警告色提示
    """

    def inner(func):
        new_content = ('<h2>'
                       '<span>Tip:【用例可持续优化】</span>'
                       '</h2>')
        for index, each in enumerate(content.split(';')):
            new_content += f'<h3 style="color: #f0bb0e">{index + 1}. {each}</h3>'
        return allure.description_html(test_description_html=new_content)(func)

    return inner


def case_priority(order=0):
    """
    用例执行的优先级, 其中 0 的优先级最高, 比负数、正数都高
    example:

        @priority(order=2)
        def test_01():
            pass

        @priority(order=1)
        def test_02():
            pass

    actual order is: test_02 -> test_01
    """

    def inner(func):
        return pytest.mark.run(order=order)(func)

    return inner



"""
@Author: 馒头 (chocolate)
@Email: neihanshenshou@163.com
@File: __init__.py
@Time: 2023/12/9 18:00
"""

from .CaseDecorator import case_attach
from .CaseDecorator import case_desc_error
from .CaseDecorator import case_desc_html
from .CaseDecorator import case_desc_ok
from .CaseDecorator import case_desc_up
from .CaseDecorator import case_feature
from .CaseDecorator import case_priority
from .CaseDecorator import case_severity
from .CaseDecorator import case_skip
from .CaseDecorator import case_skip_if
from .CaseDecorator import case_step
from .CaseDecorator import case_story
from .CaseDecorator import case_tag
from .CaseDecorator import case_title
from .CaseDecorator import fixture
from .DecoratorFormat import Decorators
from .DecoratorFormat import param_data
from .DecoratorFormat import param_file
from .DecoratorFormat import timer



"""
@Author: 馒头 (chocolate)
@Email: neihanshenshou@163.com
@File: HttpRequest.py
@Time: 2023/12/9 18:00
"""

import json as json_v2
from urllib import parse
from urllib.parse import unquote

from requests import Response
from requests import Session

from SteamedBun import logger


def __hook_response(response: Response, **kwargs):
    response.encoding = "utf-8"
    try:
        result = json_v2.dumps(response.json(), ensure_ascii=False)
    except Exception as e:
        result = response.text or e

    kwargs = {"服务code码": response.status_code, **kwargs}
    if kwargs.get("proxies"):
        del kwargs["proxies"]

    query = unquote(parse.urlparse(response.url).query or parse.urlparse(response.url).params)
    body = response.request.body
    trace_id = response.headers.get("traceid")
    try:
        body = body.decode(encoding="utf-8") if body else None
        body = json_v2.dumps(json_v2.loads(body), ensure_ascii=False)
    except Exception as e:
        print("" and e, end="")

    logger.info(f"""
    请求方法: {response.request.method}
    请求地址: {response.request.url.split("?")[0]}
    请求内容: {body or query or {} }
    请求响应: {result}
    请求时长: {response.elapsed.total_seconds()} 秒
    更多内容: {kwargs}
    TraceId: {trace_id}
        """)


def request(url,
            method="post",
            show=True,
            params=None,
            data=None,
            headers=None,
            cookies=None,
            timeout=10,
            verify=None,
            json=None,
            **kwargs):
    """

    :param url:
    :param method:
    :param show:
    :param params:
    :param data:
    :param headers:
    :param cookies:
    :param timeout:
    :param verify:
    :param json:
    :param kwargs:
    :return:
    """
    show = kwargs.get("show") or show
    with Session() as session:
        return session.request(
            method=method,
            url=url,
            params=params,
            data=data,
            headers=headers,
            cookies=cookies,
            timeout=timeout,
            verify=verify,
            json=json,
            hooks=dict(response=__hook_response) if show else None,
            **kwargs)


def get(url, show=True, params=None, headers=None, cookies=None, timeout=10, verify=None, **kwargs):
    r"""Sends a GET request.

    :param show:
    :param verify:
    :param timeout:
    :param cookies:
    :param headers:
    :param url: URL for the new :class:`Request` object.
    :param params: (optional) Dictionary, list of tuples or bytes to send
        in the query string for the :class:`Request`.
    :return: :class:`Response <Response>` object
    :rtype: requests.Response
    """

    return request(method="get",
                   url=url,
                   show=show,
                   params=params,
                   headers=headers,
                   cookies=cookies,
                   timeout=timeout,
                   verify=verify, **kwargs)


def options(url, **kwargs):
    r"""Sends an OPTIONS request.

    :param url: URL for the new :class:`Request` object.
    :return: :class:`Response <Response>` object
    :rtype: requests.Response
    """

    return request(method="options", url=url, **kwargs)


def head(url, **kwargs):
    r"""Sends a HEAD request.

    :param url: URL for the new :class:`Request` object.
    :return: :class:`Response <Response>` object
    :rtype: requests.Response
    """

    kwargs.setdefault("allow_redirects", False)
    return request(method="head", url=url, **kwargs)


def post(url,
         data=None,
         show=True,
         params=None,
         json=None,
         headers=None,
         cookies=None,
         timeout=10,
         verify=None,
         **kwargs):
    r"""Sends a POST request.

    :param show:
    :param verify:
    :param timeout:
    :param params:
    :param cookies:
    :param headers:
    :param url: URL for the new :class:`Request` object.
    :param data: (optional) Dictionary, list of tuples, bytes, or file-like
        object to send in the body of the :class:`Request`.
    :param json: (optional) A JSON serializable Python object to send in the body of the :class:`Request`.
    :return: :class:`Response <Response>` object
    :rtype: requests.Response
    """

    return request(method="post",
                   url=url,
                   show=show,
                   data=data,
                   json=json,
                   params=params,
                   headers=headers,
                   cookies=cookies,
                   timeout=timeout,
                   verify=verify,
                   **kwargs)


def put(url,
        data=None,
        show=True,
        params=None,
        json=None,
        headers=None,
        cookies=None,
        timeout=10,
        verify=None,
        **kwargs):
    r"""Sends a PUT request.

    :param data:
    :param show:
    :param params:
    :param json:
    :param verify:
    :param timeout:
    :param cookies:
    :param headers:
    :param url: URL for the new :class:`Request` object.
    :param data: (optional) Dictionary, list of tuples, bytes, or file-like
        object to send in the body of the :class:`Request`.
    :return: :class:`Response <Response>` object
    :rtype: requests.Response
    """

    return request(method="put",
                   url=url,
                   show=show,
                   data=data,
                   json=json,
                   params=params,
                   headers=headers,
                   cookies=cookies,
                   timeout=timeout,
                   verify=verify,
                   **kwargs)


def patch(url, data=None, **kwargs):
    r"""Sends a PATCH request.

    :param url: URL for the new :class:`Request` object.
    :param data: (optional) Dictionary, list of tuples, bytes, or file-like
        object to send in the body of the :class:`Request`.
    :return: :class:`Response <Response>` object
    :rtype: requests.Response
    """

    return request(method="patch", url=url, data=data, **kwargs)


def delete(url,
           data=None,
           show=True,
           params=None,
           json=None,
           headers=None,
           cookies=None,
           timeout=10,
           verify=None,
           **kwargs):
    r"""Sends a DELETE request.

    :param data:
    :param show:
    :param params:
    :param json:
    :param verify:
    :param timeout:
    :param cookies:
    :param headers:
    :param url: URL for the new :class:`Request` object.
    :return: :class:`Response <Response>` object
    :rtype: requests.Response
    """

    return request(method="delete",
                   url=url,
                   show=show,
                   data=data,
                   json=json,
                   params=params,
                   headers=headers,
                   cookies=cookies,
                   timeout=timeout,
                   verify=verify,
                   **kwargs)



"""
Exceptions that may happen in all the _Poium code.
"""


class PoiumException(Exception):
    """
    Base _Poium exception.
    """

    def __init__(self, msg=None, screen=None, stacktrace=None):
        self.msg = msg
        self.screen = screen
        self.stacktrace = stacktrace

    def __str__(self):
        exception_msg = "Message: %s\n" % self.msg
        if self.screen is not None:
            exception_msg += "Screenshot: available via screen\n"
        if self.stacktrace is not None:
            stacktrace = "\n".join(self.stacktrace)
            exception_msg += "Stacktrace:\n%s" % stacktrace
        return exception_msg


class DriverNoneException(PoiumException):
    """
    Thrown when driver is empty error
    """
    pass


class PageSelectException(PoiumException):
    """
    Thrown when using a select box error
    """
    pass


class PageElementError(PoiumException):
    """
    Raises an error using the PagElement class
    """
    pass


class FindElementTypesError(PoiumException):
    """
    Find element types Error
    """
    pass


class CSSFindElementError(PoiumException):
    """
    Find element types Error
    """
    pass


class FuncTimeoutException(Exception):
    """
    function timeout Error
    """
    pass



"""
author: 馒头
email: neihanshenshou@163.com
"""


class NumberTools:

    @staticmethod
    def point(number, rate=2):
        """
        :param number: 数字
        :param rate: 小数位
        :return point(1, 3) = 1.000
        """
        if not isinstance(number, (int, float,)):
            try:
                number = float(number)
            except Exception as e:
                if isinstance(number, str):
                    if number.isalnum():
                        number = ord(number)
                        return f"{number:.{rate}f}"
                raise "请输入数字 或者可转换为数字的字符" and e
        return f"{number:.{rate}f}"

    @staticmethod
    def percent(number, rate=2):
        """
        :param number: 数字
        :param rate: 小数位 默认为2
        :return
            point(1) = 1.00%
            point(100) = 100.00%
        """

        return f"{NumberTools.point(number=number, rate=rate)}%"



"""
@Author: 馒头 (chocolate)
@Email: neihanshenshou@163.com
@File: __init__.py
@Time: 2023/12/9 18:00
"""

import os
import time

from PIL import Image
from PIL import ImageDraw
from PIL import ImageFont

from SteamedBun import logger

current_path = os.path.abspath(__file__)
BASE_DIR = os.path.abspath(os.path.dirname(current_path) + os.path.sep)


class Cfg:
    # 报告截图路径
    report_snapshot = "."


def compress_image(filename: str):
    """
    不改变图片尺寸压缩到指定大小
    """
    im = Image.open(filename)
    im.save(filename, quality=100)


def screenshots_name(describe=None):
    """
    生成截图的名称, 主要是用于 pytest 用例中
    """
    case_path = os.environ.get('PYTEST_CURRENT_TEST')[:-7]
    this_case_name = case_path.split("/")[-1]
    now_time = int(round(time.time() * 1000))
    tmp_file_name = this_case_name + "::" + str(now_time) + ".jpg"
    print("\n")
    describe = "" if not describe else " => " + describe
    logger.info("截图 📷" + describe + " => " + tmp_file_name)
    snapshot_dir = Cfg.report_snapshot + "/"
    snapshot_name = "{path}{name}".format(path=snapshot_dir, name=tmp_file_name)
    return snapshot_name


def processing(filename: str, image="", text: str = "⊙", color: str = 'red', w=None, h=None, font_size=45):
    """
    点击截图增加水印
    :param filename: 源图片文件
    :param image: 另存为(可选)
    :param text: 水印文案
    :param color: 水印颜色
    :param w: 水印位置 宽 比
    :param h: 水印位置 高 比
    :param font_size: 水印字体大小
    :return:
    """
    font_dir = os.path.join(BASE_DIR, "font/song.ttc")
    font = ImageFont.truetype(font_dir, font_size)
    im1 = Image.open(filename)

    draw = ImageDraw.Draw(im1)
    img_width, img_height = im1.size
    x = img_width * w
    y = img_height * h
    if w is not None and h is not None:
        w = x + font_size / 2
        h = y
    draw.text(xy=(w or x, h or y), text=text, fill=color, font=font, stroke_width=2)  # 设置文字位置/内容/颜色/字体/加粗
    ImageDraw.Draw(im1)  # Just draw it!
    im1.save(image or filename)

    compress_image(filename)



"""
@Author: 馒头 (chocolate)
@Email: neihanshenshou@163.com
@File: DecoratorFormat.py
@Time: 2023/12/9 18:00
"""

import functools
import inspect
import json
import os
from pathlib import Path

import pandas
import pytest
import yaml


def timer(func):
    @functools.wraps(func)
    def inner(*args, **kwargs):
        import time
        from SteamedBun import logger

        __start_time = time.time()
        resp = func(*args, **kwargs)
        logger.info(msg=f'{func.__name__}() 共耗时: %.3f秒' % (time.time() - __start_time))
        return resp

    return inner


class Decorators:
    """"""

    @staticmethod
    def load_json_file(file_path):
        with open(file_path, 'r', encoding="utf-8") as f:
            return json.loads(f.read())

    @staticmethod
    def load_csv_file(file_path):
        try:
            csv_data = pandas.read_csv(filepath_or_buffer=file_path, dtype=object).to_dict(orient="list")
        except Exception as e:
            raise Exception(f"An exception occurred while reading a CSV file: {e}")

        # 获取所有键名
        keys = list(csv_data.keys())

        # 按索引位置组合所有列表的值（使用*解包.values()）
        value_groups = zip(*csv_data.values())

        return [dict(zip(keys, values)) for values in value_groups]

    @staticmethod
    def load_excel_file(file_path, sheet_name="Sheet1"):
        try:
            excel = pandas.read_excel(file_path, sheet_name=sheet_name, engine='openpyxl', dtype=object)
            excel_data = excel.to_dict(orient="list")
        except Exception as e:
            raise Exception(f"An exception occurred while reading a excel file: {e}")

        # 获取所有键名
        keys = list(excel_data.keys())

        # 按索引位置组合所有列表的值（使用*解包.values()）
        value_groups = zip(*excel_data.values())

        return [dict(zip(keys, values)) for values in value_groups]

    @staticmethod
    def load_yaml_file(file_path):
        with open(file_path, 'r', encoding="utf-8") as f:
            data = yaml.safe_load(f.read())
            if isinstance(data, dict):
                if 'param_data' not in data:
                    raise Exception('yaml 文件中至少要有 param_data 的 key')
                return data['param_data']
            return data

    @staticmethod
    def get_test_data_file_by_env(path, suffix='yaml'):
        # 只有 filename 不带 相应 后缀才会自动根据执行环境做自动识别
        typos = ("yaml", "json", "csv", "xlsx")
        if not any([path.endswith(_) for _ in typos]) and suffix in typos:
            # 遵循 SteamedBun 模块为任意地方可以 import 的原则，使用局部 import Config
            # if "test" in ("test", "ppe"):
            from SteamedBun import SetEnvironment
            env = SetEnvironment()
            if env.current_env not in ["tce", "ppe", "online"]:
                real_file = path + f'.test.{suffix}'
            else:
                real_file = path + f'.online.{suffix}'
            # 如果用例不区分环境，直接取对应的 json 后缀的文件
            if not os.path.exists(real_file):
                real_file = path + f'.{suffix}'
        else:
            real_file = path
        return real_file

    @staticmethod
    def _parameterized_func(case, parameter_data, enable_index=None, *args, **kwargs):
        p = []
        for index, each in enumerate(parameter_data):
            # 如果设置了 enable_index，并且没有指定当前参数化，则跳过当前
            # 主要用于调试用例时进行 debug
            if enable_index is not None and index not in enable_index:
                continue
            p.append(each)

        return pytest.mark.parametrize('param', p, *args, **kwargs)(case)

    @classmethod
    def param_file(cls, filename: str, enable_index=None, enable_envs=None,
                   disable_envs=None, suffix='yaml', sheet_name="Sheet1", *args, **kwargs):
        def inner(func):
            case_dir = Path(inspect.getfile(func)).parent
            parameter_file = str(case_dir.joinpath(filename))
            if enable_envs is not None and "test" not in enable_envs \
                    or disable_envs is not None and "test" in disable_envs:
                return pytest.mark.skip(f'Case not run in test ! 当前环境下没有对应的测试文件')
            real_file = cls.get_test_data_file_by_env(parameter_file, suffix=suffix)

            try:
                real_suffix = real_file.split(".")[-1] or suffix
                if real_suffix == 'yaml':
                    parameter_data = cls.load_yaml_file(file_path=real_file)
                elif real_suffix == 'json':
                    parameter_data = cls.load_json_file(file_path=real_file)
                elif real_suffix == "csv":
                    parameter_data = cls.load_csv_file(file_path=real_file)
                elif real_suffix == "xlsx":
                    parameter_data = cls.load_excel_file(file_path=real_file, sheet_name=sheet_name)
                else:
                    raise Exception('没有对应的类型文件')
                if not isinstance(parameter_data, (list, tuple)):
                    return pytest.mark.skip('参数化json文件 {} 不是数组！'.format(parameter_file))
                return cls._parameterized_func(func, parameter_data, enable_index, *args, **kwargs)
            except FileNotFoundError:
                from SteamedBun import SetEnvironment
                raise FileNotFoundError(f'当前环境: {SetEnvironment().current_env}, 缺少对应的参数化文件: {parameter_file}')

        return inner

    @classmethod
    def param_data(cls, parameter_data: list, enable_index=None, *args, **kwargs):
        def inner(func):
            if not isinstance(parameter_data, (list, tuple)):
                return pytest.mark.skip('参数化json数据 {} 不是数组！'.format(parameter_data))
            return cls._parameterized_func(func, parameter_data, enable_index, *args, **kwargs)

        return inner


param_file = Decorators.param_file
param_data = Decorators.param_data



"""
@Author: 馒头 (chocolate)
@Email: neihanshenshou@163.com
@File: TimeFormat.py
@Time: 2023/12/9 18:00
"""

import calendar
import time
from datetime import datetime

from dateutil import parser


class TimeFormat:
    def __init__(self, dt=None):
        dt = dt or time.time()
        if isinstance(dt, datetime):
            self.date = dt
        elif isinstance(dt, (float, int)):
            self.date = datetime.fromtimestamp(dt)
        elif isinstance(dt, str):
            self.date = parser.parse(timestr=dt)

    def __add__(self, other):
        """
            将两个datetime对象相加，需要首先将它们转换为时间间隔

            other = TimeFormat("00:30") - TimeFormat("01:30")
        :param other: 时间间隔
        :return:
        """
        self.date = other.date + self.date
        return self

    def __sub__(self, other):
        """
            将它们转换为时间间隔
        :param other: 时间间隔
        :return:
        """
        self.date -= other.date
        return self

    def __gt__(self, other):
        """
        两个对象进行比较大小, greater than
        :param other: 另一个对象
        :return:
        """
        return self.date > other.date

    def __lt__(self, other):
        """
        两个对象进行比较大小, less than
        :param other: 另一个对象
        :return:
        """
        return self.date < other.date

    @staticmethod
    def flash_back(seconds=0, minutes=0, hours=0, days=1, month=0, weeks=0, years=0) -> datetime:
        """
        时间回溯器、时光穿越机
        :param seconds: 回溯 < 0, 穿越 > 0
        :param minutes: 回溯 < 0, 穿越 > 0
        :param hours: 回溯 < 0, 穿越 > 0
        :param days: 回溯 < 0, 穿越 > 0
        :param month: 回溯 < 0, 穿越 > 0
        :param weeks: 回溯 < 0, 穿越 > 0
        :param years: 回溯 < 0, 穿越 > 0
        :return: datetime
        """
        future = 0
        if seconds:
            future += seconds
        if minutes:
            future += minutes * 60
        if hours:
            future += hours * 3600
        if days:
            future += days * 86400
        if month:
            today = datetime.today()
            month_range = calendar.monthrange(today.year, today.month)[1]
            future += month * month_range * 86400
        if weeks:
            future += weeks * 7 * 86400
        if years:
            future += years * 365 * 86400
        return datetime.fromtimestamp(int(str(int(future + time.time()))[:10]))



"""
@Author: 馒头 (chocolate)
@Email: neihanshenshou@163.com
@File: func_timeout
@Time: 2025/6/29 17:20
"""

import functools
import threading

from SteamedBun._Poium.universal.exceptions import FuncTimeoutException


def func_timeout(timeout):
    """
    A decorator to limit the execution time of a function.
    If the function's execution time exceeds the `timeout` seconds, a `FuncTimeoutException` exception will be raised.
    """

    def decorator(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            result = [None]
            exceptions = [None]

            def target():
                nonlocal result, exceptions
                try:
                    result[0] = func(*args, **kwargs)
                except Exception as e:
                    exceptions[0] = e

            thread = threading.Thread(target=target)
            thread.start()
            thread.join(timeout)

            if thread.is_alive():
                raise FuncTimeoutException(f"Function {func.__name__} timed out after {timeout} seconds")

            if exceptions[0]:
                raise exceptions[0]

            return result[0]

        return wrapper

    return decorator



"""
@Author: 馒头 (chocolate)
@Email: neihanshenshou@163.com
@File: __init__.py
@Time: 2023/12/9 18:00
"""

from . import playwright
from .config import BrowserObject
from .javascript import CSSElement
from .playwright import Locator
from .playwright import Locators
from .processing import compress_image
from .processing import processing
from .selenium import Element
from .selenium import Elements
from .webdriver import Page



"""
@Author: 馒头 (chocolate)
@Email: neihanshenshou@163.com
@File: selenium.py
@Time: 2023/12/9 18:00
"""
import os
import platform
from time import sleep, time

import allure
from selenium.common.exceptions import StaleElementReferenceException
from selenium.common.exceptions import WebDriverException
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.select import Select

from SteamedBun import logger
from SteamedBun._Poium.config import BrowserObject
from SteamedBun._Poium.universal.exceptions import DriverNoneException
from SteamedBun._Poium.universal.exceptions import FindElementTypesError
from SteamedBun._Poium.universal.exceptions import FuncTimeoutException
from SteamedBun._Poium.universal.exceptions import PageElementError
from SteamedBun._Poium.universal.func_timeout import func_timeout
from SteamedBun._Poium.universal.selector import selection_checker

# Map PageElement constructor arguments to webdriver locator enums
LOCATOR_LIST = {
    # selenium
    'css': By.CSS_SELECTOR,
    'id_': By.ID,
    'name': By.NAME,
    'xpath': By.XPATH,
    'link_text': By.LINK_TEXT,
    'partial_link_text': By.PARTIAL_LINK_TEXT,
    'tag': By.TAG_NAME,
    'class_name': By.CLASS_NAME,

}

BY_LIST = [
    # selenium
    By.CSS_SELECTOR,
    By.ID,
    By.NAME,
    By.XPATH,
    By.LINK_TEXT,
    By.PARTIAL_LINK_TEXT,
    By.TAG_NAME,
    By.CLASS_NAME
]


def _auto_screenshot(elem, context, describe=""):
    from SteamedBun import processing, case_attach

    window_size = context.get_window_size()
    window_width, window_height = window_size.get("width") - 14, window_size.get("height") - 140
    _rect: dict = elem.rect
    elem_x, elem_y = _rect.get("x", 0), _rect.get("y", 0)
    _, elem_height = _rect.get("width", 0), _rect.get("height", 0) / 4
    filename = f"{describe or 'selenium'}_screenshot_{int(time() * 1000)}.png"
    filename = os.path.join(BrowserObject.selenium_screenshot_path, filename)
    context.save_screenshot(filename=filename)
    processing(
        filename=filename,
        w=elem_x / window_width,
        h=(elem_y + elem_height) / window_height
    )
    case_attach(
        body=open(file=filename, mode="rb").read(),
        name=filename,
        attachment_type=allure.attachment_type.PNG
    )


class BasePage:
    """
    Page Object pattern.
    """

    def __init__(self, driver=None, url: str = None):
        """
        :param driver: `selenium.webdriver.WebDriver` Selenium webdriver instance
        :param url: `str`
        """
        self.driver = None
        if driver is not None:
            self.driver = driver
        else:
            ...

        if self.driver is None:
            raise DriverNoneException("driver is None, Please set selenium/appium driver.")
        self.root_uri = url if url else getattr(self.driver, 'url', None)

    def get(self, uri: str) -> None:
        """
        go to uri
        :param uri: URI to GET, based off of the root_uri attribute.
        :return:
        """
        root_uri = self.root_uri or ''
        self.driver.get(root_uri + uri)
        self.driver.implicitly_wait(5)

    def open(self, uri: str) -> None:
        """
        open uri
        :param uri:  URI to GET, based off of the root_uri attribute.
        :return:
        """
        root_uri = self.root_uri or ''
        self.driver.get(root_uri + uri)
        logger.info(f'正在进入【{uri}】网站')
        self.driver.implicitly_wait(5)

    def refresh(self):
        self.driver.refresh()
        logger.info("✅ 刷新窗口")

    def maximize_window(self):
        """
        Maximizes the current window that webdriver is using
        """
        self.driver.maximize_window()
        logger.info("✅ 窗口最大化")

    def minimize_window(self):
        """
        Invokes the window manager-specific 'minimize' operation
        """
        self.driver.minimize_window()
        logger.info("✅ 窗口最小化")

    def close(self):
        """
            Closes the current window.
        """
        self.driver.close()
        logger.info("✅ 关闭当前窗口")

    def quit(self):
        """
            Quits the driver and closes every associated window.
        """
        self.driver.quit()
        logger.info("✅ 关闭浏览器")

    def fullscreen_window(self):
        """
        Invokes the window manager-specific 'full screen' operation
        """
        self.driver.fullscreen_window()
        logger.info("✅ 窗口最大化")

    def get_screenshot_as_png(self):
        """
        Gets the screenshot of the current window as a binary data.
        """
        _ = self.driver.get_screenshot_as_png()
        logger.info("✅ 截图当前窗口 以二进制的形式")
        return _

    def get_screenshot_as_base64(self):
        """
        Gets the screenshot of the current window as a base64 encoded string
           which is useful in embedded images in HTML.
        """
        _ = self.driver.get_screenshot_as_base64()
        logger.info("✅ 截图当前窗口 以base64的形式")
        return _


class Element(object):
    """
    Returns an element object
    """

    def __init__(self,
                 selector: str = None,
                 id_: str = "",
                 name: str = "",
                 css: str = "",
                 xpath: str = "",
                 class_name: str = "",
                 timeout: int = 3,
                 describe: str = "",
                 index: int = 0,
                 **kwargs):
        self.selector = selector
        self.times = timeout
        self.index = index
        self.desc = describe
        self.exist = False

        if selector is not None:
            self.k, self.v = selection_checker(selector)
        else:
            if id_:
                kwargs['id_'] = id_
            elif name:
                kwargs['name'] = name
            elif css:
                kwargs['css'] = css
            elif xpath:
                kwargs['xpath'] = xpath
            elif class_name:
                kwargs['class_name'] = class_name
            if not kwargs:
                raise ValueError(f"Please specify a locator from {LOCATOR_LIST}")
            if len(kwargs) > 1:
                raise ValueError(f"Please specify only one locator from {LOCATOR_LIST}")
            self.kwargs = kwargs
            by, self.v = next(iter(kwargs.items()))

            self.k = LOCATOR_LIST.get(by, None)
            if self.k is None:
                raise FindElementTypesError("Element positioning of type '{}' is not supported.".format(self.k))

    def __get__(self, instance, owner):
        if instance is None:
            return None

        BrowserObject.driver = instance.driver
        return self

    def __set__(self, instance, value):
        self.__get__(instance, instance.__class__)
        self.send_keys(value)

    @func_timeout(1)
    def find_elements_timeout(self, key: str, value: str):
        try:
            return BrowserObject.driver.find_elements(key, value)
        except Exception as e:
            return [] and e

    def find(self, by: str, value: str) -> list:
        """
        Find if the element exists.
        """
        for i in range(self.times):
            try:
                elems = self.find_elements_timeout(by, value)
                break
            except FuncTimeoutException:
                sleep(1)
        else:
            elems = []

        if len(elems) == 1:
            logger.info(f"🔍 Find element: {by}={value}. {self.desc}")
        elif len(elems) > 1:
            logger.info(f"❓ Find {len(elems)} elements through: {by}={value}. {self.desc}")
        else:
            logger.warning(f"❌ Find 0 elements through: {by}={value}. {self.desc}")

        return elems

    def __get_element(self, by: str, value: str):
        """
        Judge element positioning way, and returns the element.
        """

        if by in BY_LIST:
            elem = self.find(by, value)
            if len(elem) == 0:
                self.exist = False
                return None
            else:
                self.exist = True
                elem = BrowserObject.driver.find_elements(by, value)[self.index]
        else:
            raise FindElementTypesError("Please enter the correct targeting elements")

        if BrowserObject.show is True:
            try:
                style_red = 'arguments[0].style.border="2px solid #FF0000"'
                style_blue = 'arguments[0].style.border="2px solid #00FF00"'
                style_null = 'arguments[0].style.border=""'

                BrowserObject.driver.execute_script(style_red, elem)
                sleep(0.02)
                BrowserObject.driver.execute_script(style_blue, elem)
                sleep(0.02)
                BrowserObject.driver.execute_script(style_blue, elem)
                sleep(0.01)
                BrowserObject.driver.execute_script(style_null, elem)
            except WebDriverException:
                pass

        if BrowserObject.selenium_screenshot_path:
            _auto_screenshot(elem=elem, context=BrowserObject.driver, describe=self.desc)
        return elem

    def is_exist(self) -> bool:
        """element is existed """
        self.__get_element(self.k, self.v)
        return self.exist

    def clear(self) -> None:
        """Clears the text if it's a text entry element."""
        logger.info("✅ clear.")
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception(f"❌ clear() error because of ❌ Find 0 elements through: {self.k}={self.v}. {self.desc}")
        elem.clear()

    def send_keys(self, value, clear=False, click=False) -> None:
        """
        Simulates typing into the element.
        If clear_before is True, it will clear the content before typing.
        """
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception(f"❌ send_keys() error because of Find 0 elements through: {self.k}={self.v}. {self.desc}")

        if click is True:
            elem.click()
            sleep(0.3)
            logger.info("✅ click().")
        if clear is True:
            elem.clear()
            sleep(0.3)
            logger.info("✅ clear().")
        elem.send_keys(value)
        logger.info(f"✅ send_keys('{value}').")

    def click(self) -> None:
        """
        Clicks the element.
        """
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception(f"❌ click() error because of Find 0 elements through: {self.k}={self.v}. {self.desc}")

        elem.click()
        logger.info("✅ click().")

    def submit(self):
        """
        Submits a form.
        """
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception(f"❌ submit() error because of Find 0 elements through: {self.k}={self.v}. {self.desc}")

        elem.submit()
        logger.info("✅ submit().")

    @property
    def tag_name(self) -> str:
        """This element's ``tagName`` property."""
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception(f"❌ get tag_name error because of Find 0 elements through: {self.k}={self.v}. {self.desc}")

        tag_name = elem.tag_name
        logger.info(f"✅ tag_name: {tag_name}.")
        return tag_name

    @property
    def text(self) -> str:
        """The text of the element."""
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception(f"❌ get text error because of Find 0 elements through: {self.k}={self.v}. {self.desc}")

        text = elem.text
        logger.info(f"✅ text: {text}.")
        return text

    @property
    def size(self) -> dict:
        """The size of the element."""
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception(f"❌ get size error because of Find 0 elements through: {self.k}={self.v}. {self.desc}")

        size = elem.size
        logger.info(f"✅ size: {size}.")
        return size

    def value_of_css_property(self, property_name):
        """
        The value of a CSS property
        :param property_name:
        """
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception(
                f"❌ get css property error because of Find 0 elements through: {self.k}={self.v}. {self.desc}")

        property_value = elem.value_of_css_property(property_name)
        logger.info(f"✅ value_of_css_property('{property_name}') -> {property_value}.")
        return property_value

    def get_property(self, name) -> str:
        """
        Gets the given property of the element.
        """
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception(
                f"❌ get_property('{name}') error because of Find 0 elements through: {self.k}={self.v}. {self.desc}")

        value = elem.get_property(name)
        logger.info(f"✅ get_property('{name}') -> {value}.")
        return value

    def get_attribute(self, name) -> str:
        """
        Gets the given attribute or property of the element.
        """
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception(
                f"❌ get_property('{name}') error because of Find 0 elements through: {self.k}={self.v}. {self.desc}")

        value = elem.get_attribute(name)
        logger.info(f"✅ get_property('{name}') -> {value}.")
        return value

    def is_displayed(self) -> bool:
        """Whether the element is visible to a user."""
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception(
                f"❌ is_displayed() error because of Find 0 elements through: {self.k}={self.v}. {self.desc}")

        display = elem.is_displayed()
        logger.info(f"✅ is_displayed() -> {display}.")
        return display

    def is_selected(self):
        """
        Returns whether the element is selected.

        Can be used to check if a checkbox or radio button is selected.
        """
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception(
                f"❌ is_selected() error because of Find 0 elements through: {self.k}={self.v}. {self.desc}")

        select = elem.is_selected()
        logger.info(f"✅ is_selected() -> {select}.")
        return select

    def is_enabled(self):
        """Returns whether the element is enabled."""
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception(
                f"❌ is_enabled() error because of Find 0 elements through: {self.k}={self.v}. {self.desc}")

        enable = elem.is_enabled()
        logger.info(f"✅ is_enabled() -> {enable}.")
        return enable

    def switch_to_frame(self) -> None:
        """
        selenium API
        Switches focus to the specified frame
        """
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception(
                f"❌ switch_to_frame() error because of Find 0 elements through: {self.k}={self.v}. {self.desc}")

        BrowserObject.driver.switch_to.frame(elem)
        logger.info("✅ switch_to_frame().")

    def move_to_element(self) -> None:
        """
        selenium API
        Moving the mouse to the middle of an element
        """
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception(
                f"❌ move_to_element() error because of Find 0 elements through: {self.k}={self.v}. {self.desc}")

        ActionChains(BrowserObject.driver).move_to_element(elem).perform()
        logger.info("✅ move_to_element().")

    def click_and_hold(self) -> None:
        """
        selenium API
        Holds down the left mouse button on an element.
        """
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception(
                f"❌ click_and_hod() error because of Find 0 elements through: {self.k}={self.v}. {self.desc}")

        ActionChains(BrowserObject.driver).click_and_hold(elem).perform()
        logger.info("✅ click_and_hold().")

    def double_click(self) -> None:
        """
        selenium API
        Holds down the left mouse button on an element.
        """
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception(
                f"❌ double_click() error because of Find 0 elements through: {self.k}={self.v}. {self.desc}")

        ActionChains(BrowserObject.driver).double_click(elem).perform()
        logger.info("✅ double_click().")

    def context_click(self) -> None:
        """
        selenium API
        Performs a context-click (right click) on an element.
        """
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception(
                f"❌ context_click() error because of Find 0 elements through: {self.k}={self.v}. {self.desc}")

        ActionChains(BrowserObject.driver).context_click(elem).perform()
        logger.info("✅ context_click().")

    def drag_and_drop_by_offset(self, x: int, y: int) -> None:
        """
        selenium API
        Holds down the left mouse button on the source element,
           then moves to the target offset and releases the mouse button.
        :param x: X offset to move to.
        :param y: Y offset to move to.
        """
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception(f"❌ drag_and_drop_by_offset('{x}', '{y}') error because of "
                            f"Find 0 elements through: {self.k}={self.v}. {self.desc}")

        ActionChains(BrowserObject.driver).drag_and_drop_by_offset(elem, xoffset=x, yoffset=y).perform()
        logger.info(f"✅ drag_and_drop_by_offset('{x}', '{y}').")

    def refresh_element(self, timeout: int = 5) -> None:
        """
        selenium API
        Refreshes the current page, retrieve elements.
        """
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception(f"❌ refresh_element() error because of"
                            f"Find 0 elements through: {self.k}={self.v}. {self.desc}")

        for i in range(timeout):
            if elem is not None:
                try:
                    elem
                except StaleElementReferenceException:
                    BrowserObject.driver.refresh()
                else:
                    break
            else:
                sleep(1)
        else:
            raise TimeoutError("stale element reference: element is not attached to the page document.")

    def select_by_value(self, value: str) -> None:
        """
        selenium API
        Select all options that have a value matching the argument. That is, when given "foo" this
           would select an option like:

           <option value="foo">Bar</option>

           :Args:
            - value - The value to match against

           throws NoSuchElementException If there is no option with specisied value in SELECT
        """
        select_elem = self.__get_element(self.k, self.v)
        if not select_elem:
            raise Exception(f"❌ select_by_value('{value}') error because of "
                            f"Find 0 elements through: {self.k}={self.v}. {self.desc}")

        Select(select_elem).select_by_value(value)
        logger.info(f"✅ select_by_value('{value}').")

    def select_by_index(self, index: int) -> None:
        """
        selenium API
        Select the option at the given index. This is done by examing the "index" attribute of an
           element, and not merely by counting.

           :Args:
            - index - The option at this index will be selected

           throws NoSuchElementException If there is no option with specisied index in SELECT
        """
        select_elem = self.__get_element(self.k, self.v)
        if not select_elem:
            raise Exception(f"❌ select_by_index('{index}') error because of "
                            f"Find 0 elements through: {self.k}={self.v}. {self.desc}")

        Select(select_elem).select_by_index(index)
        logger.info(f"✅ select_by_index('{index}').")

    def select_by_visible_text(self, text: str) -> None:
        """
        selenium API
        Select all options that display text matching the argument. That is, when given "Bar" this
           would select an option like:

            <option value="foo">Bar</option>

           :Args:
            - text - The visible text to match against

            throws NoSuchElementException If there is no option with specisied text in SELECT
        """
        select_elem = self.__get_element(self.k, self.v)
        if not select_elem:
            raise Exception((f"❌ select_by_visible_text('{text}') error because of "
                             f"Find 0 elements through: {self.k}={self.v}. {self.desc}"))

        Select(select_elem).select_by_visible_text(text)
        logger.info(f"✅ select_by_visible_text('{text}').")

    def input(self, text="") -> None:
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception((f"❌ input('{text}') error because of"
                             f" Find 0 elements through: {self.k}={self.v}. {self.desc}"))

        elem.send_keys(text)
        logger.info(f"🎹 input('{text}').")

    def enter(self) -> None:
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception((f"❌ enter() error because of"
                             f" Find 0 elements through: {self.k}={self.v}. {self.desc}"))

        elem.send_keys(Keys.ENTER)
        logger.info("🎹 enter.")

    def select_all(self) -> None:
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception((f"❌ select_all() error because of"
                             f" Find 0 elements through: {self.k}={self.v}. {self.desc}"))

        if platform.system().lower() == "darwin":
            elem.send_keys(Keys.COMMAND, "a")
        else:
            elem.send_keys(Keys.CONTROL, "a")
        logger.info("🎹 control + a.")

    def cut(self) -> None:
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception((f"❌ cut() error because of"
                             f" Find 0 elements through: {self.k}={self.v}. {self.desc}"))

        if platform.system().lower() == "darwin":
            elem.send_keys(Keys.COMMAND, "x")
        else:
            elem.send_keys(Keys.CONTROL, "x")
        logger.info("🎹 control + x.")

    def copy(self) -> None:
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception((f"❌ copy() error because of"
                             f" Find 0 elements through: {self.k}={self.v}. {self.desc}"))

        if platform.system().lower() == "darwin":
            elem.send_keys(Keys.COMMAND, "c")
        else:
            elem.send_keys(Keys.CONTROL, "c")
        logger.info("🎹 control + c.")

    def paste(self) -> None:
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception((f"❌ paste() error because of"
                             f" Find 0 elements through: {self.k}={self.v}. {self.desc}"))

        if platform.system().lower() == "darwin":
            elem.send_keys(Keys.COMMAND, "v")
        else:
            elem.send_keys(Keys.CONTROL, "v")
        logger.info("🎹 control + v.")

    def backspace(self) -> None:
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception((f"❌ backspace() error because of"
                             f" Find 0 elements through: {self.k}={self.v}. {self.desc}"))

        elem.send_keys(Keys.BACKSPACE)
        logger.info("🎹 backspace.")

    def delete(self) -> None:
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception((f"❌ delete() error because of"
                             f" Find 0 elements through: {self.k}={self.v}. {self.desc}"))

        elem.send_keys(Keys.DELETE)
        logger.info("🎹 delete.")

    def tab(self) -> None:
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception((f"❌ tabl() error because of"
                             f" Find 0 elements through: {self.k}={self.v}. {self.desc}"))

        elem.send_keys(Keys.TAB)
        logger.info("🎹 tab.")

    def space(self) -> None:
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception((f"❌ space() error because of"
                             f" Find 0 elements through: {self.k}={self.v}. {self.desc}"))

        elem.send_keys(Keys.SPACE)
        logger.info("🎹 space.")

    def screenshots(self, filename=None):
        """
        selenium API
        Saves a screenshots of the current element to a PNG image file
        :param filename: The file name
        """

        if filename is None:
            filename = str(time()).split(".")[0] + ".png"
        elem = self.__get_element(self.k, self.v)
        if not elem:
            raise Exception((f"❌ screenshot() error because of"
                             f" Find 0 elements through: {self.k}={self.v}. {self.desc}"))

        elem.screenshot(filename)
        logger.info("✅ screenshot.")


class Elements(object):
    """
    Returns a set of element objects
    """

    def __init__(self,
                 selector: str = None,
                 id_: str = "",
                 name: str = "",
                 css: str = "",
                 xpath: str = "",
                 class_name: str = "",
                 context: bool = False,
                 describe: str = "",
                 timeout: int = 3,
                 **kwargs):
        self.desc = describe
        self.times = timeout
        if selector is not None:
            self.k, self.v = selection_checker(selector)
        else:
            if id_:
                kwargs['id_'] = id_
            elif name:
                kwargs['name'] = name
            elif css:
                kwargs['css'] = css
            elif xpath:
                kwargs['xpath'] = xpath
            elif class_name:
                kwargs['class_name'] = class_name
            if not kwargs:
                raise ValueError("Please specify a locator")
            if len(kwargs) > 1:
                raise ValueError("Please specify only one locator")
            by, self.v = next(iter(kwargs.items()))

            self.k = LOCATOR_LIST.get(by, None)
            if self.k is None:
                raise FindElementTypesError("Element positioning of type '{}' is not supported.".format(self.k))

        self.has_context = bool(context)

    def find(self, context):
        for i in range(self.times):
            elems = context.find_elements(self.k, self.v)
            if len(elems) > 0:
                break
            else:
                sleep(1)
        else:
            elems = []

        for elem in elems:
            try:
                style_red = 'arguments[0].style.border="2px solid #FF0000"'
                style_blue = 'arguments[0].style.border="2px solid #00FF00"'
                style_null = 'arguments[0].style.border=""'

                context.execute_script(style_red, elem)
                sleep(0.02)
                context.execute_script(style_blue, elem)
                sleep(0.02)
                context.execute_script(style_blue, elem)
                sleep(0.01)
                context.execute_script(style_null, elem)
            except WebDriverException:
                pass

            if BrowserObject.selenium_screenshot_path:
                _auto_screenshot(elem=elem, context=context, describe=self.desc)

        logger.info(f"✨ Find {len(elems)} elements through: {self.k}={self.v}. {self.desc}")
        return elems

    def __get__(self, instance, owner, context=None):
        if not instance:
            return None

        if not context and self.has_context:
            return lambda ctx: self.__get__(instance, owner, context=ctx)

        if not context:
            context = instance.driver

        return self.find(context)

    def __set__(self, instance, value):
        if self.has_context:
            raise PageElementError("Sorry, the set descriptor doesn't support elements with context.")
        elems = self.__get__(instance, instance.__class__)
        if not elems:
            raise PageElementError("Can't set value, no elements found")
        [elem.send_keys(value) for elem in elems]



"""
@Author: 馒头 (chocolate)
@Email: neihanshenshou@163.com
@File: setup.py
@Time: 2025/01/23 23:36
"""

from setuptools import find_packages
from setuptools import setup

__version__ = "2.1.9"

# 包目录下资源
PackageData = [
    "font/song.ttc",
    "browser_session.yaml",
    "config.ini"
]

# 依赖的三方库
DependentPackage = [
    "allure-pytest==2.13.2",
    "colorama==0.4.6",
    "NumPy==1.23.5",
    "openpyxl==3.1.0",
    "pandas>=2.0.3",
    "Pillow==9.5.0",
    "playwright==1.44.0",
    "pymysql==1.1.1",
    "python-dateutil==2.8.2",
    "pytest==7.3.2",
    "pytest-ordering==0.6",
    "pytest-xdist==3.5.0",
    "PyYAML==6.0",
    "requests==2.30.0",
    "retry==0.9.2",
    "selenium==4.4.3",
    "setuptools<=60.2.0",
    "SteamedBun-Uninstall==1.0.0",
    "urllib3==1.26.12"
]

setup(
    name="SteamedBun",
    author="馒头",
    author_email="neihanshenshou@163.com",
    description="馒头的第三方库",
    long_description=open(file="README.md", encoding="utf-8", mode="r").read(),
    long_description_content_type="text/markdown",
    packages=find_packages(),
    include_package_data=True,
    package_data={"": PackageData},
    version=__version__,
    install_requires=DependentPackage,
    license="Apache License 2.0",
    license_file="LICENSE",
    platforms=["MacOS、Window"],
    fullname="馒头大人",
    url="https://github.com/neihanshenshou/SteamedBun",
    python_requires=">=3.5",  # 支持的Python版本范围
)



"""
@Author: 馒头 (chocolate)
@Email: neihanshenshou@163.com
@File: FileOperateFormat.py
@Time: 2023/12/9 18:00
"""

import json
import os
import time
from typing import Union, Any, Literal

import pandas
import yaml


class FileNames:
    """
    文件名称
    """
    LogFile = f"{time.strftime('%F')}.log"
    ScreenShotFile = f"{time.strftime('%F')}.png"
    ExcelFile = f"{time.strftime('%F')}.xlsx"
    CsvFile = f"{time.strftime('%F')}.csv"
    YamlFile = f"{time.strftime('%F')}.yaml"
    JsonFile = f"{time.strftime('%F')}.json"
    TxtFile = f"{time.strftime('%F')}.txt"


class FileOperate:

    @staticmethod
    def _is_json_serializable(data: str):
        """
        判断一个对象 是否能够json格式化
        :param data: 字符串
        :return: bool
        """
        try:
            json.loads(data)
            return True
        except Exception as e:
            return False and e

    @staticmethod
    def _is_yaml_serializable(data: str):
        """
        判断一个对象 是否能够yaml格式化
        :param data: 字符串
        :return: bool
        """
        try:
            yaml.safe_load(data)
            return True
        except Exception as e:
            return False and e

    @staticmethod
    def read_excel(filename, sheet_name='Sheet1'):
        """
            读取Excel文件内容
        :param filename: filename: Excel文件名称
        :param sheet_name: Excel文件sheet表名
        :return: excel.to_dict(orient="list") or excel.to_dict(orient="dict")
        """
        excel = pandas.read_excel(filename, sheet_name=sheet_name, engine='openpyxl')
        return excel

    @staticmethod
    def write_excel(data: dict, filename, sheet_name='Sheet1', show=True):
        """
            写入Excel文件内容
        :param data: {"a": [1, 2, 3], "b": [4, 5, 6]}
        :param filename: Excel文件名称
        :param sheet_name: Excel文件sheet表名
        :param show: 打印结束标志
        :return: None
        """
        df1 = pandas.DataFrame(data)
        df1.to_excel(filename, sheet_name=sheet_name, index=False, engine='openpyxl')
        if show:
            print(f"✅ excel: {filename} 写入完成")

    @staticmethod
    def write_excel_with_many_sheets(filename, data):

        # Create a dictionary of DataFrames
        dfs = {sheet_name: pandas.DataFrame(sheet_data) for sheet_name, sheet_data in data.items()}

        # Write to Excel with multiple sheets
        with pandas.ExcelWriter(filename) as writer:
            for sheet_name, df in dfs.items():
                df.to_excel(writer, sheet_name=sheet_name, index=False)

        print(f'Multiple sheets written to {filename}')

    @staticmethod
    def read_csv(filename, orient: Literal["dict", "list", "series", "split", "tight", "records", "index"] = "list"):
        """
            读取csv文件内容
        :param filename: filename: csv文件名称
        :param orient:
        :return: csv_data type: list or dict
        """
        csv_obj = pandas.read_csv(filepath_or_buffer=filename)
        csv_data = csv_obj.to_dict(orient=orient)
        return csv_data

    @staticmethod
    def write_csv(data: dict, filename, encoding="utf-8", show=True):
        """
            写入Excel文件内容
        :param data: {"a": [1, 2, 3], "b": [4, 5, 6]}
        :param filename: csv 文件名称
        :param encoding: csv 文件编码
        :param show: 打印结束标志
        :return: None
        """
        df1 = pandas.DataFrame(data)
        df1.to_csv(path_or_buf=filename, index=False, encoding=encoding)
        if show:
            print(f"✅ csv: {filename} 写入完成")

    @staticmethod
    def read_file(filename: str = '', key: str = None, mode='r', encoding: Any = "utf-8", jsonify=False, yamlify=False):
        """
        读取文件, 若文件内容可以json格式化、yaml格式化 则返回dict, 反之返回字符串
        :param filename: 文件名称
        :param key: 键值
        :param encoding: 编码
        :param jsonify: json格式化开关
        :param yamlify: yaml格式化开关
        :param mode: 读取模式
        :return: dict or str
        """
        with open(file=filename, mode=mode, encoding=encoding) as f:
            _data = f.read()

        if len(_data) >= 1:
            if FileOperate._is_json_serializable(data=_data) and jsonify:
                js = json.loads(_data)
                return js[key] if key else js
        if FileOperate._is_yaml_serializable(data=_data) and yamlify:
            y = yaml.safe_load(_data)
            return y[key] if key else y

        return _data

    @staticmethod
    def read_json(filename: str, key: str = None, encoding: Any = "utf-8"):
        return FileOperate.read_file(filename=filename, key=key, encoding=encoding, jsonify=True)

    @staticmethod
    def read_yaml(filename: str, key: str = None, encoding: Any = "utf-8"):
        return FileOperate.read_file(filename=filename, key=key, encoding=encoding, yamlify=True)

    @staticmethod
    def read_txt(filename: str, encoding: Any = "utf-8"):
        return FileOperate.read_file(filename=filename, encoding=encoding)

    @staticmethod
    def write_file(filename: str, data: Union[list, dict, str, bytes], mode='w', encoding: Any = "utf-8", show=True):
        """
        写入文件
        :param filename: 文件名称
        :param data: 内容
        :param encoding: 编码格式
        :param mode: 读写模式
        :param show: 打印结束标志
        :return: None

        example:
                FileOperate.write_file(
                    filename="demo.csv",
                    data="a,b,c\n1,2,3"
                )

                FileOperate.write_file(
                    filename="demo.json",
                    data={}
                )

                FileOperate.write_file(
                    filename="demo.yaml",
                    data=[{"name": "a"}, {"name": "b"}]
                )
                and so on ...

        """
        if filename.endswith('json'):
            data = json.dumps(data, ensure_ascii=False)
        if filename.endswith('yaml'):
            data = yaml.safe_dump(data, allow_unicode=True)
        if isinstance(data, (list, dict)):
            data = json.dumps(data, ensure_ascii=False, indent=4)
        with open(file=filename, mode=mode, encoding=encoding) as f:
            f.write(data)
        if show:
            print(f"✅ 文件: {filename} 写入完成!")

    @staticmethod
    def traverse_directory(directory, exclude="venv, idea", show=False):
        """
        遍历指定目录及其子目录中的所有文件
        :param directory: 需要遍历的目录
        :param exclude: 需要排除的目录
        :param show: 是否展示遍历的文件
        """
        result = []
        excludes = [each.strip() for each in exclude.split(",")]
        for root, dirs, files in os.walk(directory):
            for file in files:
                file_path = os.path.join(root, file)
                if all([each not in file_path for each in excludes]):
                    if show:
                        print(file_path)
                    result.append(file_path)
        return result



"""
@Author: 馒头 (chocolate)
@Email: neihanshenshou@163.com
@File: ExpectFormat.py
@Time: 2023/12/9 18:00
"""


class ExpectFormat:

    @staticmethod
    def should_contains(member, container, desc=None):
        """
        Examples
        Args1:
            first: int = 1
            second: int = [1, 2] or (1, 2) or {1} or {1: 2}
            desc: will return True
        Args2:
            first: list = [1, 2]
            second: int = [1, 2, 3] or (3, 1, 2) or {3, 2, 1} or {1: 2, 3: 5, 2: 4}
            desc: will return True
        """
        from SteamedBun import logger
        if isinstance(member, (str, int, float)):
            if member not in container:
                raise AttributeError(f"""
                member: {member}
                container: {container}
                desc: {desc or "期望member是container的子集, 实际与期望结果不一致"}
                """)
            else:
                logger.info(f"""
                member: {member}
                container: {container}
                desc: {desc or f"期望member({type(member)})是container({type(container)})的子集, 实际与期望结果一致"}
                """)
        if isinstance(member, (list, dict, set, tuple)):
            for m in member:
                ExpectFormat.should_contains(member=m, container=container)
            logger.info(f"""
            member: {member}
            container: {container}
            desc: {desc or "期望member是container的子集, 实际与期望结果一致"}
            """)

    @staticmethod
    def should_equal(first, second, desc: str = ""):
        """
        Examples
        Args1:
            first: int = 1
            second: int = 1
            desc: will return True

        Args2:
            first: str = 'abc'
            second: str = 'abc'
            desc: will return True

        Args3:
            first: list = ['abc']
            second: str = ['abc', 1.3]
            desc: will return False

        Args4:
            first: dict = {"key1": "value1", "key2": "value2"}
            second: dict = {"key2": "value2", "ke1y": "value1"}
            desc: will return False

        Returns:

        """
        from SteamedBun import logger
        if first != second:
            raise AssertionError(
                f"""
                first: {first}
                second: {second}
                desc: 实际{desc}与预期不相同
                """
            )
        else:
            logger.info(f"""
            first: {first}
            second: {second}
            desc: {desc} 校验通过
            """)

    @staticmethod
    def should_not_equal(first, second, desc: str = ""):
        """
        Examples
        Args1:
            first: int = 1
            second: int = 1
            desc: will return False

        Args2:
            first: str = 'abc'
            second: str = 'abc'
            desc: will return False

        Args3:
            first: list = ['abc']
            second: str = ['abc', 1.3]
            desc: will return True

        Args4:
            first: dict = {"key1": "value1", "key2": "value2"}
            second: dict = {"key2": "value2", "ke1y": "value1"}
            desc: will return True

        Returns:

        """
        from SteamedBun import logger
        if first == second:
            raise AssertionError(
                f"""
                first: {first}
                second: {second}
                desc: 实际{desc}与预期不相同
                """
            )
        else:
            logger.info(f"""
            first: {first}
            second: {second}
            desc: {desc} 校验通过
            """)



"""
@Author: 馒头 (chocolate)
@Email: neihanshenshou@163.com
@File: __init__.py
@Time: 2023/12/9 18:00
"""
from .HttpRequest import delete
from .HttpRequest import get
from .HttpRequest import head
from .HttpRequest import options
from .HttpRequest import patch
from .HttpRequest import post
from .HttpRequest import put
from .HttpRequest import request
